unit AliasDBDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, AliasDialog, Db, Grids, DBGrids, ZAbstractRODataset, ZDataset;

type
  TAliasDB = class(TAlias)
    DBGrid1: TDBGrid;
    DataSource: TDataSource;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure AliasQueryAfterScroll(DataSet: TDataSet);
    procedure DBGrid1CellClick(Column: TColumn);
    procedure PlayerAliasChange(Sender: TObject);
    procedure PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    reentered: Boolean;
    lastsearch: string;
	FQuery: TZReadOnlyQuery;
  public
    { Public declarations }
  end;

var
  AliasDB1: TAliasDB;

implementation

uses TornMain, Utility;

{$R *.DFM}

procedure TAliasDB.FormCreate(Sender: TObject);
begin
  FQuery := TZReadOnlyQuery.Create(Self);
  FQuery.Connection := MainForm.ZConnection;
  case TornForm.SystemType of
    stNexus: FQuery.SQL.Add('SELECT Alias AS Player_Alias, '''' AS First_Name, '''' AS Last_Name, Button_ID AS User_ID FROM members');
    stNexusWithZeon:
      if MainForm.HeliosType < 47 then  // This is the old schema, with QRCode in demographics.customer table.
        FQuery.SQL.Add('SELECT M.Alias AS Player_Alias, C.First_Name AS First_Name, C.Last_Name AS Last_Name, C.QRCode AS User_ID ' +
          'FROM members M left join demographics.customers C on C.Customer_ID = M.member_ID ' +
          'WHERE SUBSTRING(C.QRCode, 1, 5) <> ''00005'' ORDER BY Alias')
      else  // This is the new schema, with QRCode in members table.
        FQuery.SQL.Add('SELECT M.Alias AS Player_Alias, '''' AS First_Name, '''' AS Last_Name, M.QRCode AS User_ID ' +
          'FROM members M ' +
          'WHERE SUBSTRING(M.QRCode, 1, 5) <> ''00005'' ORDER BY Alias');
    stAcacia: FQuery.SQL.Add('SELECT Player_Alias, First_Name, Last_Name, User_ID FROM MEMBERS WHERE User_ID <> ''''');
    stLaserforce: ;
  end;
  FQuery.Open;
  Datasource.DataSet := FQuery;
  DBGrid1.DataSource := DataSource;
  FQuery.AfterScroll := AliasQueryAfterScroll;

  DBGrid1.Columns[1].Visible := TornForm.SystemType = stAcacia;
  DBGrid1.Columns[2].Visible := TornForm.SystemType = stAcacia;

  if TornForm.SystemType = stNexusWithZeon then begin
    PlayerAlias.Width := PlayerAlias.Width - 60;
    ButtonID.Left := ButtonID.Left - 60;
    ButtonID.Width := ButtonID.Width + 60;
  end;

  lastsearch := '%'
end;

procedure TAliasDB.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FQuery.AfterScroll := nil;
  try
    FQuery.Free;
  except
    ;
  end;
end;

procedure TAliasDB.FormResize(Sender: TObject);
begin
  if TornForm.SystemType in [stAcacia, stNexusWithZeon] then begin
    if DBGrid1.Columns.Count > 0 then DBGrid1.Columns[0].Width := Trunc((DBGrid1.Width - 25) * 0.4);
    if DBGrid1.Columns.Count > 1 then DBGrid1.Columns[1].Width := Trunc((DBGrid1.Width - 25) * 0.3);
    if DBGrid1.Columns.Count > 2 then DBGrid1.Columns[2].Width := Trunc((DBGrid1.Width - 25) * 0.3);
  end else
    if DBGrid1.Columns.Count > 0 then DBGrid1.Columns[0].Width := DBGrid1.Width - 25;
end;

procedure TAliasDB.PlayerAliasChange(Sender: TObject);
var s: string;
begin
  if reentered then Exit;
  reentered := True;
  try
    Application.ProcessMessages;  // if the user has typed several keystrokes into the edit box, we don't want to do several SQL queries.  Ensure that PlayerAlias.Text contains up-to-date info.
    s := '%' + UpperCase(Copy(PlayerAlias.Text, 1, PlayerAlias.SelStart)) + '%';
    if lastsearch = s then Exit;  // if we've already done this search (because the user has typed several keystrokes into the edit box), skip it.
    lastsearch := s;

    Datasource.DataSet := nil;
    try
      FQuery.Free;
    except
      ;
    end;

    FQuery := TZReadOnlyQuery.Create(Self);
    FQuery.Connection := MainForm.ZConnection;
    case TornForm.SystemType of  // Note SQL injections in the LIKE clauses of these. Parameterise? Sanitise?
      stNexus: FQuery.SQL.Add('SELECT Alias AS Player_Alias, '''' AS First_Name, '''' AS Last_Name, Button_ID AS User_ID FROM members WHERE UPPER(Alias) LIKE ''' + s + ''' ');
      stNexusWithZeon: FQuery.SQL.Add('SELECT M.Alias AS Player_Alias, '''' AS First_Name, '''' AS Last_Name, M.QRCode AS User_ID ' +
        'FROM members M ' +
        'WHERE SUBSTRING(M.QRCode, 1, 5) <> ''00005'' AND UPPER(Alias) LIKE ''' + s + ''' ORDER BY Alias');
      stAcacia: FQuery.SQL.Add('SELECT Player_Alias, First_Name, Last_Name, User_ID FROM MEMBERS' +
                    ' WHERE (UPPER(Player_Alias) LIKE "' + s + '" ' +
                    'OR UPPER(First_Name) || '' '' || UPPER(Last_Name) LIKE "' + s + '") ' +
                    'AND User_ID <> ''''');
      stLaserforce: ;
    end;
    FQuery.Open;
    Datasource.DataSet := FQuery;
    FQuery.AfterScroll := AliasQueryAfterScroll;
    AliasQueryAfterScroll(FQuery);
  finally
    reentered := False;
  end;
end;

procedure TAliasDB.PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_UP then begin
    FQuery.Prior;
    Key := 0;
  end;
  if Key = VK_DOWN then begin
    FQuery.Next;
    Key := 0;
  end;

  if Key in [VK_BACK, VK_LEFT] then begin
    PlayerAlias.SelStart := PlayerAlias.SelStart - 1;
    PlayerAlias.SelLength := 999;
    Key := 0;
  end;
  if Key = VK_RIGHT then begin
    PlayerAlias.SelStart := PlayerAlias.SelStart + 1;
    PlayerAlias.SelLength := 999;
    Key := 0;
  end;
end;

procedure TAliasDB.AliasQueryAfterScroll(DataSet: TDataSet);
var i: Integer;
begin
  if FQuery.Eof then Exit;

  reentered := True;

  i := PlayerAlias.SelStart;
  if UpperCase(Copy(PlayerAlias.Text, 1, i)) = UpperCase(Copy(NonNull(FQuery.FieldValues['Player_Alias']), 1, i)) then  // if the unselected characters of what the user typed match the first few characters of the player alias
    PlayerAlias.Text := NonNull(FQuery.FieldValues['Player_Alias'])
  else if UpperCase(Copy(PlayerAlias.Text, 1, i)) = UpperCase(Copy(NonNull(FQuery.FieldValues['First_Name'] + ' ' + FQuery.FieldValues['Last_Name']), 1, i)) then  // if the unselected characters of what the user typed match the first few characters of the player name
    PlayerAlias.Text := NonNull(FQuery.FieldValues['First_Name']) + ' ' + NonNull(FQuery.FieldValues['Last_Name'])
  else
    PlayerAlias.Text := Copy(PlayerAlias.Text, 1, i);

  PlayerAlias.SelStart := i;
  PlayerAlias.SelLength := 999;

  ButtonID.Text := NonNull(FQuery.FieldValues['User_ID']);

  reentered := False;
end;

procedure TAliasDB.DBGrid1CellClick(Column: TColumn);
begin
  AliasQueryAfterScroll(FQuery);
end;

end.
