unit AliasLaserforceDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry,
  StdCtrls, Grids, ZoomReport, AliasDialog;

type
  TAliasLaserforce = class(TAlias)
    Grid1: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PlayerAliasChange(Sender: TObject);
    procedure PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AliasQueryAfterScroll;
    procedure Grid1SelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
  private
    reentered: Boolean;
    lastsearch: string;
    aliasautocomplete: Integer;
    query: TZoomReport;

    procedure DoQuery(s: string);
  public
    { Public declarations }
  end;

var
  Alias: TAliasLaserforce;

implementation

uses TornMain, Utility;

{$R *.DFM}

procedure TAliasLaserforce.DoQuery(s: string);

  function IDExists(id: string): Boolean;
  var i: Integer;
  begin
    Result := False;
    i := 0;
    while not Result and (i < Grid1.RowCount) do begin
      if Grid1.Cells[3, i] = id then
        Result := True;
      Inc(i);
    end;
  end;

var g, q: Integer;
begin
  query := GetTable(TornForm.Prefs.DBServerIP, LASERFORCE_PORT,
                    'SELECT M.ref, M.surname, M.givenNames, M.codename, ' +
                    'cast(C.region as varchar) + ''-'' + cast(C.site as varchar) + ''-'' + cast(M.id as varchar) as [ID] ' +
                    'FROM Member M ' +
                    'LEFT JOIN Centre C ON C.ref = M.centre ' +
                    'WHERE M.surname LIKE ''' + s + '%'' OR M.givenNames LIKE ''' + s + '%'' OR M.codename LIKE ''%' + s + '%'' ' +
                    'ORDER BY M.codename, [ID]');

  g := 0;

  Grid1.RowCount := query.Rows.Count + 1;
  Grid1.Cells[0, 0] := 'Code Name';
  Grid1.Cells[1, 0] := 'First Name';
  Grid1.Cells[2, 0] := 'Last Name';
  Grid1.Cells[3, 0] := 'ID';

  for q := 0 to query.Rows.Count - 1 do begin
//    if not IDExists(query.RowCol(q, 'ID').Text) then begin
      Grid1.Cells[0, g] := query.RowCol(q, 'codename').Text;
      Grid1.Cells[1, g] := query.RowCol(q, 'givenNames').Text;
      Grid1.Cells[2, g] := query.RowCol(q, 'surname').Text;
      Grid1.Cells[3, g] := query.RowCol(q, 'ID').Text;

      Inc(g);
//    end;
  end;

  Grid1.RowCount := g;
end;

procedure TAliasLaserforce.FormCreate(Sender: TObject);
var i: Integer;
    sel: TGridRect;
    MyRegistry: TRegistry;
begin
    MyRegistry := TRegistry.Create;  // open the Registry
    try
        MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
        if not MyRegistry.KeyExists('Software\DougWare\Torn') then
            MyRegistry.CreateKey('Software\DougWare\Torn');  // registry key for Torn doesn't exist yet; create it.

        MyRegistry.OpenKey('Software\DougWare\Torn', True);

        if MyRegistry.ValueExists('AliasAutocomplete') and (MyRegistry.GetDataType('AliasAutocomplete') = rdInteger) then
          aliasautocomplete := MyRegistry.ReadInteger('AliasAutocomplete')
        else
          aliasautocomplete := 0;

        MyRegistry.CloseKey;
    finally
        MyRegistry.Free;
    end;

  if aliasautocomplete = 0 then begin
    query := GetTable(TornForm.Prefs.DBServerIP, LASERFORCE_PORT,
                      'SELECT M.ref, M.surname, M.givenNames, M.codename, ' +
                      'cast(C.region as varchar) + ''-'' + cast(C.site as varchar) + ''-'' + cast(M.id as varchar) as [ID] ' +
                      'FROM Member M ' +
                      'LEFT JOIN Centre C ON C.ref = M.centre ' +
                      'ORDER BY M.codename, [ID]');

    Grid1.RowCount := query.Rows.Count + 1;
    Grid1.Cells[0, 0] := 'Code Name';
    Grid1.Cells[1, 0] := 'First Name';
    Grid1.Cells[2, 0] := 'Last Name';
    Grid1.Cells[3, 0] := 'ID';

    for i := 0 to query.Rows.Count - 1 do begin
      Grid1.Cells[0, i + 1] := query.RowCol(i, 'codename').Text;
      Grid1.Cells[1, i + 1] := query.RowCol(i, 'givenNames').Text;
      Grid1.Cells[2, i + 1] := query.RowCol(i, 'surname').Text;
      Grid1.Cells[3, i + 1] := query.RowCol(i, 'ID').Text;
    end;

    lastsearch := '';

    sel.Left := 0;
    sel.Right := 3;
    sel.Top := 1;
    sel.Bottom := 1;
    Grid1.Selection := sel;
  end else
    query := nil;
end;

procedure TAliasLaserforce.FormDestroy(Sender: TObject);
begin
  if Assigned(query) then
    query.Free;
end;

procedure TAliasLaserforce.FormResize(Sender: TObject);
begin
  if Grid1.ColCount > 0 then Grid1.ColWidths[0] := Trunc((Grid1.Width - 25) * 0.3);
  if Grid1.ColCount > 1 then Grid1.ColWidths[1] := Trunc((Grid1.Width - 25) * 0.25);
  if Grid1.ColCount > 2 then Grid1.ColWidths[2] := Trunc((Grid1.Width - 25) * 0.25);
  if Grid1.ColCount > 3 then Grid1.ColWidths[3] := Trunc((Grid1.Width - 25) * 0.2);
end;

procedure TAliasLaserforce.PlayerAliasChange(Sender: TObject);
var g, q: Integer;
    s: string;
begin
  if reentered then Exit;
  reentered := True;
  try
    Application.ProcessMessages;  // if the user has typed several keystrokes into the edit box, we don't want to do several SQL queries.  Ensure that PlayerAlias.Text contains up-to-date info.
    s := UpperCase(Copy(PlayerAlias.Text, 1, PlayerAlias.SelStart));
    if lastsearch = s then Exit;  // if we've already done this search (because the user has typed several keystrokes into the edit box), skip it.
    lastsearch := s;

    if Length(s) >= aliasautocomplete then
      DoQuery(s);

    if (aliasautocomplete = 0) or (Length(s) >= aliasautocomplete) then begin
      Grid1.RowCount := query.Rows.Count + 1;

      g := 1;
      for q := 0 to query.Rows.Count - 1 do
        if (s = '') or
           (Pos(s, UpperCase(query.RowCol(q, 'codename').Text)) > 0) or
           (Pos(s, UpperCase(query.RowCol(q, 'givenNames').Text + ' ' + query.RowCol(q, 'surname').Text)) > 0) or
           (Pos(s, UpperCase(query.RowCol(q, 'surname').Text)) > 0) or
           (Pos(s, UpperCase(query.RowCol(q, 'ID').Text)) > 0) then begin
          Grid1.Cells[0, g] := query.RowCol(q, 'codename').Text;
          Grid1.Cells[1, g] := query.RowCol(q, 'givenNames').Text;
          Grid1.Cells[2, g] := query.RowCol(q, 'surname').Text;
          Grid1.Cells[3, g] := query.RowCol(q, 'ID').Text;
          Inc(g);
        end;

      Grid1.RowCount := g;
      AliasQueryAfterScroll;
    end;
  finally
    reentered := False;
  end;
end;

procedure TAliasLaserforce.PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var sel: TGridRect;
begin
  if Key = VK_UP then begin
    sel.Left := 0;
    sel.Right := 3;
    sel.Top := Grid1.Selection.Top - 1;
    sel.Bottom := Grid1.Selection.Bottom - 1;
    Grid1.Selection := sel;
    Key := 0;
  end;
  if Key = VK_DOWN then begin
    sel.Left := 0;
    sel.Right := 3;
    sel.Top := Grid1.Selection.Top + 1;
    sel.Bottom := Grid1.Selection.Bottom + 1;
    Grid1.Selection := sel;
    Key := 0;
  end;

  if Key in [VK_BACK, VK_LEFT] then begin
    PlayerAlias.SelStart := PlayerAlias.SelStart - 1;
    PlayerAlias.SelLength := 999;
    Key := 0;
  end;
  if Key = VK_RIGHT then begin
    PlayerAlias.SelStart := PlayerAlias.SelStart + 1;
    PlayerAlias.SelLength := 999;
    Key := 0;
  end;
end;

procedure TAliasLaserforce.AliasQueryAfterScroll;
var i: Integer;
begin  reentered := True;

  i := PlayerAlias.SelStart;
  if (Grid1.RowCount > 1) and (UpperCase(Copy(PlayerAlias.Text, 1, i)) = UpperCase(Copy(Grid1.Cells[0, 1], 1, i))) then  // if the unselected characters of what the user typed match the first few characters of the player alias
    PlayerAlias.Text := Grid1.Cells[0, 1]
  else if (Grid1.RowCount > 1) and (UpperCase(Copy(PlayerAlias.Text, 1, i)) = UpperCase(Copy(Grid1.Cells[1, 1] + ' ' + Grid1.Cells[2, 1], 1, i))) then  // if the unselected characters of what the user typed match the first few characters of the player name
    PlayerAlias.Text := Grid1.Cells[1, 1] + ' ' + Grid1.Cells[2, 1]
  else
    PlayerAlias.Text := Copy(PlayerAlias.Text, 1, i);

  PlayerAlias.SelStart := i;
  PlayerAlias.SelLength := 999;

  if (Grid1.RowCount > 0) then
    ButtonID.Text := NonNull(Grid1.Cells[3, 1]);

  reentered := False;
end;

procedure TAliasLaserforce.Grid1SelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  reentered := True;
  try
    if ARow > 0 then begin
      PlayerAlias.Text := Grid1.Cells[0, ARow];
      ButtonID.Text    := Grid1.Cells[3, ARow];
    end;
  finally
    reentered := False;
  end;
end;

end.
