unit FixtureDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Games, League, ZoomReport, HillClimb, Spin, ComCtrls;

type
  TTeamOffset = record
    Team: Integer;
    Points: Double;
    Score: Integer;
  end;

  TFixtureForm = class(TForm)
    VersusImage: TImage;
    BottomPanel: TPanel;
    SendTo: TPanel;
    Label7: TLabel;
    ToScreen: TCheckBox;
    ToScoreboard: TCheckBox;
    ToPrinter: TCheckBox;
    Portrait: TRadioButton;
    Landscape: TRadioButton;
    ToHtml: TCheckBox;
    ToTsv: TCheckBox;
    OKButton: TButton;
    CancelButton: TButton;
    Generate: TButton;
    Stop: TButton;
    Label3: TLabel;
    GamesPerTeamPerWeek: TSpinEdit;
    Label2: TLabel;
    MiddlePanel: TPanel;
    Label1: TLabel;
    RightPanel: TPanel;
    Label4: TLabel;
    Weeks: TSpinEdit;
    PerWeekLabel: TLabel;
    Score: TLabel;
    Date: TDateTimePicker;
    Time: TDateTimePicker;
    DateLabel: TLabel;
    TimeLabel: TLabel;
    Label5: TLabel;
    Panel1: TPanel;
    ReportFixtures: TCheckBox;
    ReportWho: TCheckBox;
    ReportTeamLists: TCheckBox;
    Label6: TLabel;
    Label8: TLabel;
    ColourPanel: TPanel;
    CheckOrange: TCheckBox;
    CheckRed: TCheckBox;
    CheckBlue: TCheckBox;
    CheckGreen: TCheckBox;
    CheckYellow: TCheckBox;
    CheckPurple: TCheckBox;
    CheckPink: TCheckBox;
    CheckCyan: TCheckBox;
    Referees: TCheckBox;
    CheckWhite: TCheckBox;
    CheckBlack: TCheckBox;
    CheckFire: TCheckBox;
    CheckIce: TCheckBox;
    CheckEarth: TCheckBox;
    CheckCrystal: TCheckBox;
    CheckRainbow: TCheckBox;
    CheckCops: TCheckBox;
    Label9: TLabel;
    Splitter1: TSplitter;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure GenerateClick(Sender: TObject);
    procedure StopClick(Sender: TObject);
    procedure WeeksChange(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
  private
    FReports: TZoomReports;
    FTeamOffsets: array of TTeamOffset;  // This is effectively a team ladder, to determine what order teams go into the report in.
    FPlayed: TPlays;
    FAveragePlayed: Double;
    FStop: Boolean;
    FColourCount: Integer;
    FTeamsCount: Integer;
    FColours: array [TNexusColour] of TNexusColour;  // What colours has the user told us to create fixtures for?
    FHill: THillClimber;

    procedure BuildTeamsCounts(hill: THillClimber);
    procedure BuildFixtureReports(hill: THillClimber);
    procedure BuildTeamLists;
    function FindCheck(tag: Integer): TCheckbox;
    function CalcScore(hill: THillClimber; games: TGames; referee: TReferee): Int64;
  public
    GameList: TGameList;
    LeagueList: TLeagueList;

    procedure Initialise;

    property Reports: TZoomReports read FReports;
  end;

var
  FMaxPlays: Integer;  // The largest number of times a team plays another team.

implementation

uses TornMain, Math, utility;

{$R *.DFM}

procedure TFixtureForm.FormCreate(Sender: TObject);
begin
  FReports := nil;
  FHill := THillClimber.Create;
end;

procedure TFixtureForm.FormDestroy(Sender: TObject);
begin
  FReports.Free;
  FHill.Free;
end;

procedure TFixtureForm.Initialise;
var
  i, j: Integer;
  temp: TTeamOffset;
  team1: Integer;
  colours: array [TNexusColour] of Integer;
begin
  SetLength(FTeamOffsets, LeagueList.Count);
  SetLength(FPlayed, LeagueList.Count);
  for i := Low(FPlayed) to High(FPlayed) do
    SetLength(FPlayed[i], LeagueList.Count);

  for i := 0 to Length(FTeamOffsets) - 1 do begin  // Calculate a total score for each league team.
    FTeamOffsets[i].Team := i;

    FTeamOffsets[i].Score := 0;
    for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
      team1 := GameList[j].Teams.Find(LeagueList[i].TeamID);
      if team1 > -1 then begin
        FTeamOffsets[i].Points := FTeamOffsets[i].Points + GameList[j].Teams[team1].Points;
        Inc(FTeamOffsets[i].Score, GameList[j].Teams[team1].Score);
      end;
    end;
  end;  // for each team in LeagueList

  for i := 0 to Length(FTeamOffsets) - 1 do  // Bubblesort FTeamOffsets.
    for j := i to Length(FTeamOffsets) - 1 do
      if (FTeamOffsets[i].Points < FTeamOffsets[j].Points) or (FTeamOffsets[i].Points = FTeamOffsets[j].Points) and (FTeamOffsets[i].Score < FTeamOffsets[j].Score) then begin  // Swap the two entries.
        temp.Team := FTeamOffsets[j].Team;
        temp.Points := FTeamOffsets[j].Points;
        temp.Score := FTeamOffsets[j].Score;
        FTeamOffsets[j].Team := FTeamOffsets[i].Team;
        FTeamOffsets[j].Points := FTeamOffsets[i].Points;
        FTeamOffsets[j].Score := FTeamOffsets[i].Score;
        FTeamOffsets[i].Team := temp.Team;
        FTeamOffsets[i].Points := temp.Points;
        FTeamOffsets[i].Score := temp.Score;
      end;

  for i := Low(colours) to High(colours) do
    colours[i] := 0;

  for i := 0 to GameList.Count - 1 do
    for j := 0 to GameList[i].Teams.Count - 1 do
      Inc(colours[GameList[i].Teams[j].NexusColour]);

  for i := 0 to ColourPanel.ControlCount - 1 do
    if (ColourPanel.Controls[i].Tag < 0) and (colours[-1 - ColourPanel.Controls[i].Tag] > 0) then
      TCheckBox(ColourPanel.Controls[i]).Checked := True;

  if CheckWhite.Checked or CheckBlack.Checked or CheckFire.Checked or CheckIce.Checked or
     CheckEarth.Checked or CheckCrystal.Checked or CheckRainbow.Checked or CheckCops.Checked then
    ColourPanel.Width := 153
  else
    ColourPanel.Width := 80;
end;

procedure TFixtureForm.FormShow(Sender: TObject);
var
  i: Integer;
  check: TCheckbox;
begin
  Date.Date := Trunc(Now);

  if not Assigned(FReports) then
    FReports := TZoomReports.Create
  else
    FReports.Clear;

  if Length(FHill.Games) > 0 then
    BuildFixtureReports(FHill);
  BuildTeamsCounts(FHill);
  FReports.Show(VersusImage.Canvas, 1, 1, VersusImage.Width - 2, VersusImage.Height - 2);

  // Add a checkbox for each team.
  for i := 0 to Length(FTeamOffsets) - 1 do begin
    check := FindCheck(FTeamOffsets[i].Team);
    if not Assigned(check) then
      check := TCheckBox.Create(MiddlePanel);

    check.Parent := MiddlePanel;
    check.Left := 16;
    check.Top := i * 16 + 24;
    check.Caption := LeagueList[FTeamOffsets[i].Team].BuildTeamName;
    check.Width := Canvas.TextWidth(check.Caption) + 20;
    check.Tag := FTeamOffsets[i].Team;
    check.Checked := True;
  end;
end;

procedure TeamsCountsPaintBar(report: TZoomReport; row, col: Integer; canvas: TCanvas; r: TRect; color, barcolor: TColor);

  function ToInt(s: string; def: Integer): Integer;
  var plus: Integer;
  begin
    plus := Pos('+', s);
    if plus = 0 then
      Result := StrToIntDef(s, def)
    else
      Result := StrToIntDef(Copy(s, 1, plus - 1), def) + StrToIntDef(Trim(Copy(s, plus + 1, 3)), 0);
  end;

var
  points: array [0..2] of TPoint;
  fill: Real;
begin
  if col = row + 1 then begin // We are on the diagonal.
    canvas.Brush.Color := barcolor;
    canvas.Pen.Color := barcolor;
    canvas.Pen.Width := 0;

    // Draw bottom left triangle.
    // [0]: bottom left         [1]: top left               [2]: bottom right
    points[0].y := r.Bottom;    points[1].y := r.Top + 3;   points[2].y := r.Bottom;
    points[0].x := r.Left + 1;  points[1].x := r.Left + 1;  points[2].x := r.Right - 2;
    canvas.Polygon(points);

    // Draw top right triangle.
    // [0]: top right          [1]: top left               [2]: bottom right
    points[0].y := r.Top + 1;  points[1].y := r.Top + 1;   points[2].y := r.Bottom - 2;
    points[0].x := r.Right;    points[1].x := r.Left + 3;  points[2].x := r.Right;
    canvas.Polygon(points);
  end else if (col = 0) or (col = report.Columns.Count - 1)  then
    // No shading for cells in the first or last column.
  else begin
    // paint dark bar
    canvas.Pen.Color := barcolor;
    canvas.Pen.Width := 0;
    canvas.Brush.Color := barcolor;

    fill := ToInt(report.Rows[row][col].Text, 0) / FMaxPlays;

    if fill > 0 then
      canvas.Rectangle(r.Left + 1, r.Top + 1, r.Left + Round((r.Right - r.Left) * fill), r.Bottom);
  end;
end;

// Build an n x n grid showing how many times each team has played against each other team.
procedure TFixtureForm.BuildTeamsCounts(hill: THillClimber);
var
  i, j, k: Integer;
  report: TZoomReport;
  count, total, pluscount, plustotal: Integer;  // How many times did we play a particular team?
  s: string;
begin
  FMaxPlays := 1;
  report := FReports.Add('Number of times played', 'Team', 'left');

  for i := 0 to Length(FTeamOffsets) - 1 do  // Create a column for each League team.
    report.Columns.Add(LeagueList[FTeamOffsets[i].Team].BuildTeamName, 'center');

  report.Columns.Add('Games', 'right');

  for i := 0 to Length(FTeamOffsets) - 1 do begin // Create a row for each League team.
    report.Rows.Add(TZRow.Create);

    report.Rows[i].Add(LeagueList[FTeamOffsets[i].Team].BuildTeamName);  // set up team name

    total := 0;
    plustotal := 0;
    for j := 0 to Length(FTeamOffsets) - 1 do begin
      if i = j then
        report.Rows[i].Add('', clGray)
      else begin
        count := 0;

        for k := 0 to GameList.Count - 1 do  // Roll through all the games...
          if (GameList[k].Teams.Find(LeagueList[FTeamOffsets[i].Team].TeamID) > -1) and
             (GameList[k].Teams.Find(LeagueList[FTeamOffsets[j].Team].TeamID) > -1) then begin // ...looking for ones featuring these teams.
            Inc(count);
            Inc(total);
          end;

        if count = 0 then
          s := ''
        else
          s := IntToStr(count);

        FPlayed[FTeamOffsets[i].Team, FTeamOffsets[j].Team] := count;

        if Assigned(hill) and (Length(hill.Plays) > FTeamOffsets[i].Team) and (Length(hill.Plays) > FTeamOffsets[j].Team) and
           (hill.Plays[FTeamOffsets[i].Team, FTeamOffsets[j].Team] > 0) then begin
          pluscount := hill.Plays[FTeamOffsets[i].Team, FTeamOffsets[j].Team];
          s := s + '+' + IntToStr(pluscount);
          Inc(plustotal, pluscount);
          FMaxPlays := Max(FMaxPlays, count + pluscount);
        end else
          FMaxPlays := Max(FMaxPlays, count);

        report.Rows[i].Add(s);
      end;  // if i<>j
      report.Rows[i][report.Rows[i].Count - 1].BarCell := report.Rows[i].Count - 1;  // Gotta set a BarCell so that the custom paint will get called.
    end;

    s := IntToStr(total div 2);
    if (plustotal > 0) and (hill.TeamsPerGame > 1) then
      s := s + '+' + IntToStr(plustotal div (hill.TeamsPerGame - 1));

    report.Rows[i].Add(s);
  end;  // for i

  report.OnPaintBar := TeamsCountsPaintBar;
  if Assigned(hill) and (Length(hill.Plays) > 0) then
    report.Description := 'This grid shows the number of times teams have already played each other, plus the number of times they will play each other in the fixtures shown above.';
end;

procedure TFixtureForm.BuildFixtureReports(hill: THillClimber);
var
  report: TZoomReport;
  i, j, k: Integer;
  s: string;
  color: TColor;
begin
  // List of games: one row per game; one column per colour.
  report := nil;
  for j := 0 to hill.Weeks - 1 do begin
    report := FReports.Add('Fixture for ' + FormatDateTime(ShortDateFormat, Trunc(Date.Date) + j * 7), 'Game', 'left');

    for k := 0 to hill.TeamsPerGame - 1 do
      report.Columns.Add(NexusColourNames[FColours[k]], 'left');

    if hill.Referees then
      report.Columns.Add('Referee', 'left');

    for i := 0 to hill.GamesPerWeek - 1 do begin
      report.Rows.Add(TZRow.Create);
      report.Rows[i].Add(FormatDateTime(ShortTimeFormat, Frac(Time.Time) + (i mod hill.GamesPerWeek) / 4 / 24));

      for k := 0 to hill.TeamsPerGame - 1 do
        report.Rows[i].Add(LeagueList[hill.Games[i + j * hill.GamesPerWeek, k] - 1].BuildTeamName, NexusColors[FColours[k]]);

      if hill.Referees then
        report.Rows[i].Add(LeagueList[hill.Referee[i] - 1].BuildTeamName);
    end;
  end;

  if hill.Referees then
    report.Description := 'Above is a list of games, showing which teams play in each game. The last column shows which team should supply referees for each game. Below is the same information presented as a grid: each row is a team; each column is a game.'
  else
    report.Description := 'Above is a list of games, showing which teams play in each game. Below is the same information presented as a grid: each row is a team; each column is a game.';

  // Games grid: one row per team; one column per game.
  report := FReports.Add('Fixtures', 'Team', 'right');
  for i := Low(hill.Games) to High(hill.Games) do
    if (hill.Weeks > 1) and (i mod hill.GamesPerWeek = 0) then
      report.Columns.Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat,
        Trunc(Date.Date) + (i div hill.GamesPerWeek) * 7 + Frac(Time.Time) + (i mod hill.GamesPerWeek) / 4 / 24), 'center')
    else
      report.Columns.Add(FormatDateTime(ShortTimeFormat, Frac(Time.Time) + (i mod hill.GamesPerWeek) / 4 / 24), 'center');

  for i := 0 to High(FTeamOffsets) do
  begin
    report.Rows.Add(TZRow.Create);
    report.Rows[i].Add(LeagueList[FTeamOffsets[i].Team].BuildTeamName);

    for j := Low(hill.Games) to High(hill.Games) do begin
      s := '';
      color := clNone;

      for k := 0 to hill.TeamsPerGame - 1 do
        if hill.Games[j, k] - 1 = FTeamOffsets[i].Team then begin
          s := s + NexusColourAbbrevs[FColours[k]];
          color := NexusColors[FColours[k]];
        end;

      if hill.Referees and (hill.Referee[j] - 1 = FTeamOffsets[i].Team) then begin
        s := s + 'ref';
        color := clNone;
      end;

      if s = '' then
        report.Rows[i].Add(' ')
      else
        report.Rows[i].Add(s, color);
    end;
  end;
end;

procedure TFixtureForm.BuildTeamLists;
var
  report: TZoomReport;
  i, j: Integer;
  s: string;
begin
  // Team list: one row per team.
  report := FReports.Add('Team List', 'Team,Players', 'left,left');
  for i := 0 to High(FTeamOffsets) do
  begin
    report.Rows.Add(TZRow.Create);
    report.Rows[i].Add(LeagueList[FTeamOffsets[i].Team].BuildTeamName);
    s := '';
    for j := 0 to LeagueList[FTeamOffsets[i].Team].Players.Count - 1 do begin
      s := s + LeagueList[FTeamOffsets[i].Team].Players[j].Name;
      if LeagueList[FTeamOffsets[i].Team].Players[j].Comment <> '' then
        s := s + ' (' + LeagueList[FTeamOffsets[i].Team].Players[j].Comment + ')';
      if j < LeagueList[FTeamOffsets[i].Team].players.Count - 1 then
        s := s + ', ';
    end;
    report.Rows[i].Add(s);
  end;
end;

function TFixtureForm.FindCheck(tag: Integer): TCheckbox;
var
  i: Integer;
begin
  Result := nil;
  i := 0;
  while not Assigned(Result) and (i < MiddlePanel.ControlCount) do begin
    if (MiddlePanel.Controls[i] is TCheckbox) and (MiddlePanel.Controls[i].Tag = tag) then
      Result := TCheckBox(MiddlePanel.Controls[i]);
    Inc(i);
  end;
end;

function TFixtureForm.CalcScore(hill: THillClimber; games: TGames; referee: TReferee): Int64;
var
  plays: TPlays;
  colours: TColoursCount;

  procedure CalcPlays(games: TGames; firstgame, lastgame: Integer);
  var
    game, colour, colour2, team1, team2, top: Integer;
  begin
    SetLength(plays, hill.NumTeams);  // set the dynamic array to have the correct number of teams
    for team1 := Low(plays) to High(plays) do begin
      SetLength(plays[team1], hill.NumTeams);  // set each row to have the correct number of teams
      for team2 := Low(plays[team1]) to High(plays[team1]) do
        plays[team1, team2] := 0;
    end;

    for colour := Low(colours) to High(colours) do
      for team1 := Low(colours[colour]) to High(colours[colour]) do
        colours[colour, team1] := 0;

    if hill.Referees then
      top := High(colours) - 1  // The last "colour" is how many times they referee -- skip it.
    else
      top := High(colours);

    for colour := Low(colours) to top do
      for game := firstgame to lastgame do begin
        Inc(colours[colour, games[game, colour] - 1]);

        for colour2 := Low(colours) to top do
          if colour <> colour2 then
            Inc(plays[games[game, colour] - 1, games[game, colour2] - 1]);  // mark them as playing each other.
      end;
  end;

var
  game, week, colour, team1, team2, team3: Integer;
  nocolourpenalty: Integer;
  averageplays: Real;
begin
//  averageplays := Length(games) * hill.TeamsPerGame * 2 / hill.NumTeams / (hill.NumTeams - 1); // this is the average number of times each team plays each other team
  averageplays := Length(games) * hill.TeamsPerGame / FTeamsCount / (FTeamsCount - 1); // this is the average number of times each team plays each other team
//  averageplays := FAveragePlayed;

  Result := 0;
  for team1 := Low(hill.Plays) to High(hill.Plays) do  // score teams for how many times they play each other
    for team2 := team1 + 1 to High(hill.Plays[team1]) do
      Inc(Result, Round(Power(hill.Plays[team1, team2] + FPlayed[team1, team2] - averageplays - FAveragePlayed, 4) * 20));

  for game := Low(games) to High(games) do  // score teams for playing themselves
    for team1 := Low(games[game]) to high(games[game]) do
      for team2 := team1 + 1 to High(games[game]) do
        if games[game, team1] = games[game, team2] then
          Inc(Result, 10000000);

  if hill.Referees then
    for game := Low(games) to High(games) do
      for team1 := Low(games[game]) to high(games[game]) do begin
        if games[game, team1] = referee[game] then // score teams for playing in games they referee
          Inc(Result, 3);
        if (game > Low(games)) and (games[game - 1, team1] = referee[game]) then // score for playing the game before they ref
          Inc(Result, 1);
        if (game < High(games)) and (games[game + 1, team1] = referee[game]) then  // score for playing the game after they ref
          Inc(Result, 1);
      end;

  nocolourpenalty := Round((High(games) - Low(games) + 1) / hill.NumTeams / hill.Weeks);

  if hill.Referees then
    SetLength(colours, 4)
  else
    SetLength(colours, 3);

  for team1 := Low(colours) to high(colours) do
    SetLength(colours[team1], hill.NumTeams);  // set each row to have the correct number of teams

  for week := 0 to hill.Weeks - 1 do begin
    CalcPlays(games, week * hill.NumGames div hill.Weeks, (week + 1) * hill.NumGames div hill.Weeks - 1);

    for team1 := Low(plays) to High(plays) do  // score teams for how many times they play each other this week
      for team2 := team1 + 1 to High(plays[team1]) do
        Inc(Result, Round(Power(plays[team1, team2] - (averageplays / hill.Weeks), 4) * 40));

    for colour := Low(colours) to high(colours) do
      for team1 := Low(colours[colour]) to High(colours[colour]) do  // score teams for how many times they play each colour
        Inc(Result, Max(Abs(colours[colour, team1] - nocolourpenalty), 0));
  end;

  for game := 0 to High(games) - 1 do  // for each consecutive pair of games
    if (game + 1) mod (High(games) div hill.Weeks) <> 0 then  // (not counting breaks from one week to the next),
      for team1 := 0 to hill.TeamsPerGame - 1 do
        for team2 := 0 to hill.TeamsPerGame - 1 do
          if games[game, team1] = games[game + 1, team2] then begin // if the same team appears in consecutive games,
            Inc(Result, 800);  // apply back-to-back penalty.

            if game < High(games) - 2 then  // If they have three in a row,
              for team3 := 0 to hill.TeamsPerGame - 1 do
                if games[game, team1] = games[game + 2, team3] then
                  Inc(Result, 1000);  // moar penalty.
          end;
end;

procedure TFixtureForm.FormResize(Sender: TObject);
begin
  VersusImage.Picture.Bitmap.Height := VersusImage.Height;
  VersusImage.Picture.Bitmap.Width  := VersusImage.Width;

  if Assigned(FReports) then begin
    VersusImage.Canvas.Brush.Color := FReports.BackgroundColor;
    VersusImage.Canvas.FillRect(Rect(0, 0, VersusImage.Width, VersusImage.Height));
    FReports.Show(VersusImage.Canvas, 1, 1, VersusImage.Width - 2, VersusImage.Height - 2);
  end;
end;

procedure TFixtureForm.GenerateClick(Sender: TObject);

  // Overwrite auto-generated grid with one featuring just the selected teams.
  procedure SetupIncomplete;
  var
    week, game, gamespot, control, occurrence: Integer;
  begin
    game := 0;
    gamespot := 0;
    for week := 1 to FHill.Weeks do
      for control := 0 to MiddlePanel.ControlCount - 1 do
        if (MiddlePanel.Controls[control] is TCheckbox) and TCheckbox(MiddlePanel.Controls[control]).Checked then
          for occurrence := 0 to FHill.GamesPerTeamPerWeek - 1 do begin
            FHill.Games[game, gamespot] := MiddlePanel.Controls[control].Tag + 1;
            Inc(gamespot);
            if gamespot = FHill.TeamsPerGame then begin
              gamespot := 0;
              Inc(game);
            end;
          end;
  end;

var
  i, j: Integer;
begin
  // Figure out which colours the user has ticked (and how many).
  FColourCount := 0;
  for i := 0 to ColourPanel.ControlCount - 1 do
    if (ColourPanel.Controls[i].Tag < 0) and TCheckBox(ColourPanel.Controls[i]).Checked then begin
      FColours[FColourCount] := -1 - ColourPanel.Controls[i].Tag;
      Inc(FColourCount);
    end;

  if FColourCount < 2 then begin
    ShowMessage('Please tick more colours before generating fixtures.');
    Exit;
  end;

  FHill.GamesPerTeamPerWeek := GamesPerTeamPerWeek.Value;
  FHill.Weeks := Weeks.Value;
  FHill.NumTeams := LeagueList.Count;
  FHill.Referees := Referees.Checked;
  FHill.TeamsPerGame := FColourCount;

  FTeamsCount := 0;
  FAveragePlayed := 0;
  for i := 0 to MiddlePanel.ControlCount - 1 do
    if (MiddlePanel.Controls[i] is TCheckbox) and TCheckbox(MiddlePanel.Controls[i]).Checked then begin
      Inc(FTeamsCount);
      for j := 0 to MiddlePanel.ControlCount - 1 do
        if (i <> j) and (MiddlePanel.Controls[j] is TCheckbox) and TCheckbox(MiddlePanel.Controls[j]).Checked then
          FAveragePlayed := FAveragePlayed + FPlayed[MiddlePanel.Controls[i].Tag, MiddlePanel.Controls[j].Tag];
    end;

  if FTeamsCount > 1 then
    FAveragePlayed := FAveragePlayed / FTeamsCount / (FTeamsCount - 1);

  FHill.NumGames := (FTeamsCount * FHill.GamesPerTeamPerWeek + (FHill.TeamsPerGame - 1)) div FHill.TeamsPerGame * FHill.Weeks;
  FHill.Setup;
  if FTeamsCount <> LeagueList.Count then  // If not all teams are selected,
    SetupIncomplete;

  FStop := False;
  FHill.CalcScore := CalcScore;
  Screen.Cursor := crAppStart;
  CancelButton.Enabled := False;
  Generate.Enabled := False;
  Stop.Enabled := True;
  try
    repeat
      FHill.Go(stTeam, 10000);
      Score.Caption := 'Score: ' + IntToStr(FHill.LastScore);
      FReports.Clear;
      BuildFixtureReports(FHill);
      //BuildTeamsCounts(hill);  // Don't show this during, because it's inaccurate -- it shows the Plays for the current experimental grid, not for the current best grid.
      FReports.Show(VersusImage.Canvas, 1, 1, VersusImage.Width - 2, VersusImage.Height - 2);
      Application.ProcessMessages;
    until (FHill.LastScore = 0) or FStop;
  finally
    Screen.Cursor := crDefault;
    CancelButton.Enabled := True;
    Generate.Enabled := True;
    Stop.Enabled := False;
  end;
  FReports.Clear;
  FHill.CalcPlays(FHill.Games);
  BuildFixtureReports(FHill);
  BuildTeamsCounts(FHill);
  FReports.Show(VersusImage.Canvas, 1, 1, VersusImage.Width - 2, VersusImage.Height - 2);
  OKButton.Enabled := True;
end;

procedure TFixtureForm.StopClick(Sender: TObject);
begin
  FStop := True;
end;

procedure TFixtureForm.WeeksChange(Sender: TObject);
begin
  PerWeeklabel.Visible := Weeks.Value <> 1;
//  DateLabel.Visible := Weeks.Value <> 1;
//  TimeLabel.Visible := Weeks.Value <> 1;
//  Date.Visible := Weeks.Value <> 1;
//  Time.Visible := Weeks.Value <> 1;
end;

procedure TFixtureForm.OKButtonClick(Sender: TObject);
begin
  if (Reports.Count > 2) and not ReportWho.Checked then
    Reports.Delete(Reports.Count - 1);

  if ReportTeamLists.Checked then
    BuildTeamLists;
end;

end.
