unit Games;

interface

uses Classes, Graphics;

type TNexusColour = -1..15;
const NexusColors: array[TNexusColour] of TColor = (clNone, $02A0A0FF, $02FFD0A0, $02A0FFA0, $0290FFFF, $02FFA0C0, $02F0A0FF, $02FFFFA0, $02A0D0FF, $02FFFFFF, $02909090, $0290B0FF, $02FFE0E0, $0280D0D0, $02F0C0FF, $02E0E0E0, $02FFC0B0);
//nst NexusColors: array[TNexusColour] of TColor = (clNone, $02B0B0FF, $02FFD0A0, $02A0FFA0, $0290FFFF, $02FF60D0, $02F0A0FF, $02FFFFA0, $02A0D0FF);
const NexusColourNames: array[TNexusColour] of string = ('', 'Red',    'Blue',    'Green',   'Yellow',  'Purple',  'Pink',    'Cyan',    'Orange',  'White',   'Black',   'Fire',    'Ice',     'Earth',   'Crystal', 'Rainbow', 'Cops' );
const NexusColourAbbrevs: array[TNexusColour] of string = ('', 'R',    'B',       'G',       'Y',       'Pu',      'Pi',      'Cy',      'Or',      'W',       'Bk',      'F',       'I',       'E',       'Cr',      'Ra',      'Co' );
const NUM_COLOURS = 16;

type
//    TPackColour = (pcNocolour, pcRed, pcGreen, pcBlue, pcYellow);

    TTeam = class  // This is a team within a game, not a team within a league (TLeagueTeam).
    private
        FTeamID: Integer;
        FColour: string;
        FNexusColour: TNexusColour;
        FScore,
        FAdjustment: Integer;
        FPoints,
        FPointsAdjustment : Double;

        procedure SetColour(const Value: string);
        procedure SetNexusColour(const Value: TNexusColour);
    public
        Constructor Create;

        property TeamID:    Integer read FTeamID     write FTeamID;
        property Colour:    string  read FColour     write SetColour;
        property NexusColour: TNexusColour read FNexusColour write SetNexusColour;
        property Score:     Integer read FScore      write FScore;
        property Points:    Double  read FPoints     write FPoints;
        property Adjustment:Integer read FAdjustment write FAdjustment;
        property PointsAdjustment: Double read FPointsAdjustment write FPointsAdjustment;
    end;

    TTeamList = class(TList)  // of TTeam
    private
        procedure Put(Index: Integer; Team: TTeam);
        function  Get(Index: Integer): TTeam;
        function Compare(Item1, Item2: TTeam): Integer;
    public
        Backwards: Boolean;  // True if victory points for first are less than victory points for second.

        function  Add(Team: TTeam): TTeam; overload;
        function  Add: TTeam; overload;

        procedure Clear; override;
        function  Find(TeamID: Integer): Integer;
        function  Force(TeamID: Integer): Integer;
        procedure Sort;

        property Items[Index: Integer]: TTeam read Get write Put; default;
    end;

    TPlayer = class   // This is a player within a game, not a player within a league (TLeaguePlayer).
    private
        FTeamID: Integer;
        FPlayerID: string;
        FPack: string;
        FScore,
        FRank,
        FHitsBy,
        FHitsOn,
        FBaseHits,
        FBaseDestroys: Integer;
        FColour: TNexusColour;
    public
        property TeamID:    Integer read FTeamID    write FTeamID;
        property PlayerID:  string  read FPlayerID  write FPlayerID;
        property Pack:      string  read FPack      write FPack;
        property Score:     Integer read FScore     write FScore;
        property Rank:      Integer read FRank      write FRank;
        property HitsBy:    Integer read FHitsBy    write FHitsBy;
        property HitsOn:    Integer read FHitsOn    write FHitsOn;
        property BaseHits:  Integer read FbaseHits  write FBaseHits;
        property BaseDestroys: Integer read FBaseDestroys write FBaseDestroys;
        property Colour:    TNexusColour read FColour write FColour;
    end;

    TPlayerList = class(TList)  // of TPlayer
    private
        procedure Put(Index: Integer; Player: TPlayer);
        function  Get(Index: Integer): TPlayer;
    public
        function  Add(Player: TPlayer): TPlayer; overload;
        function  Add: TPlayer; overload;

        procedure Clear; override;
        function  Find(PlayerID: string): Integer;
        function  Force(PlayerID: string): Integer;
        function  FindPack(PackID: string): Integer;

        property Items[Index: Integer]: TPlayer read Get write Put; default;
    end;

    TGame = class
    private
        FTitle: string;
        FGameTime: TDateTime;
        FTeams: TTeamList;
        FPlayers: TPlayerList;
        FHits: Integer;  // Total number of hits scored in the game
        FSecret: Boolean;  // True e.g. for finals games that should not be published on the web yet.
    public
        Constructor Create;
        Destructor Destroy; override;

        function TotalScore: Integer;
        function Count: Integer;

        property Title: string read FTitle write FTitle;
        property GameTime: TDateTime read FGameTime write FGameTime;
        property Teams: TTeamList read FTeams;
        property Players: TPlayerList read FPlayers;
        property Hits: Integer read FHits write FHits;
        property Secret: boolean read FSecret write FSecret;
    end;

    TGameList = class(TList)
    private
        procedure Put(Index: Integer; Game: TGame);
        function  Get(Index: Integer): TGame;
    public
        function  Add(Game: TGame): TGame; overload;
        function  Add: TGame; overload;

        procedure Flush;
        function  Find(GameTime: TDateTime): Integer;
        function  Force(GameTime: TDateTime): Integer;
        procedure Sort; overload;

        property Items[Index: Integer]: TGame read Get write Put; default;
    end;

    TServerGame = class
    private
        FGame_ID: Integer;
        FDescription: string;
        FGameTime: TDateTime;
        FGame: TGame;  // The league game that corresponds to this server game.
    public
        property Game_ID: Integer read FGame_ID write FGame_ID;
        property Description: string read FDescription write FDescription;
        property GameTime: TDateTime read FGameTime write FGameTime;
        property Game: TGame read FGame write FGame;
    end;

    TServerGameList = class(TList)
    private
        procedure Put(Index: Integer; ServerGame: TServerGame);
        function  Get(Index: Integer): TServerGame;
    public
        function  Add(ServerGame: TServerGame): TServerGame; overload;
        function  Add: TServerGame; overload;
        procedure Clear; override;
        procedure Sort; overload;
        function Find(time: TDateTime): TServerGame; overload;
        function Find(gameid: Integer): TServerGame; overload;

        property Items[Index: Integer]: TServerGame read Get write Put; default;
    end;

function StringToNexusColour(s: string): TNexusColour;

implementation

uses Lists, SysUtils;

function StringToNexusColour(s: string): TNexusColour;
var
  colour: string;
  i: integer;
begin
  if Length(s) = 0 then begin Result := -1; Exit; end;
  colour := UpperCase(s) + ' ';

  Result := -1;
  for i := Low(TNexusColour) to High(TNexusColour) do
    if Pos(UpperCase(NexusColourNames[i]), colour) > 0 then Result := i;

  if Result = -1 then
    case UpperCase(colour)[1] of
      'R': Result := 0;
      'B': Result := 1;
      'G': Result := 2;
      'Y': Result := 3;
    end;
end;

{ TTeam }

constructor TTeam.Create;
begin
  FTeamID := -1;
  FColour := '';
  FNexusColour := -1;
  FScore := 0;
  FPoints := 0;
  FAdjustment := 0;
  FPointsAdjustment := 0;
end;

procedure TTeam.SetColour(const Value: string);
var i: TNexusColour;
begin
  FColour := Value;

  if FNexusColour = -1 then
    for i := 0 to 15 do
      if Value = NexusColourNames[i] then
        FNexusColour := i;
end;

procedure TTeam.SetNexusColour(const Value: TNexusColour);
begin
  FNexusColour := Value;
  if FColour = '' then
    FColour := NexusColourNames[Value];
end;

{ TTeamList }

procedure TTeamList.Put(Index: Integer; Team: TTeam);
begin
  inherited Put(Index, Team);
end;

function TTeamList.Get(Index: Integer): TTeam;
begin
  Result := inherited Get(Index);
end;

function TTeamList.Add(Team: TTeam): TTeam;
begin
  inherited Add(Team);
  Result := Team;
end;

function TTeamList.Add: TTeam;
begin
  Result := Add(TTeam.Create);
end;

procedure TTeamList.Clear;
var i: Integer;
begin
  for i := 0 to Count - 1 do
    if Assigned(Self[i]) then begin
      Self[i].Free;
      Self[i] := nil;
    end;

  inherited;
end;

function TTeamList.Find(TeamID: Integer): Integer;
begin
  Result := Count - 1;
  while Result > -1 do begin
    if Self[Result].TeamID = TeamID then Break;
    Dec(Result);
  end;
end;

function TTeamList.Force(TeamID: Integer): Integer;
begin
  Result := Find(TeamID);
  if Result = -1 then begin
    Result := Count;
    Add(TTeam.Create);
    Self[Result].TeamID := TeamID;
  end;
end;

function TTeamList.Compare(Item1, Item2: TTeam): Integer;
begin
  Result := Sign(Item2.Points - Item1.Points);

  if Backwards then Result := -Result;

  if Result = 0 then
    Result := Item2.Score - Item1.Score;
end;

// Bubblesort the list, using the given Compare function.
// Partly nicked from Classes.pas TList.Sort and QuickSort, to work around a bug in Delphi's QuickSort.
procedure TTeamList.Sort;
var I, J: Integer;
begin
  for I := 0 to Count - 1 do
    for J := I to Count - 1 do
      if Compare(Items[I], Items[J]) > 0 then
        Exchange(I, J);
end;

{ TPlayerList }

procedure TPlayerList.Put(Index: Integer; Player: TPlayer);
begin
  inherited Put(Index, Player);
end;

function TPlayerList.Get(Index: Integer): TPlayer;
begin
  Result := inherited Get(Index);
end;

function TPlayerList.Add(Player: TPlayer): TPlayer;
begin
  inherited Add(Player);
  Result := Player;
end;

function TPlayerList.Add: TPlayer;
begin
  Result := Add(TPlayer.Create);
end;

procedure TPlayerList.Clear;
var i: Integer;
begin
  for i := 0 to Count - 1 do
    if Assigned(Self[i]) then begin
      Self[i].Free;
      Self[i] := nil;
    end;

  inherited;
end;

function TPlayerList.Find(PlayerID: string): Integer;
begin
  Result := Count - 1;
  while Result > -1 do begin
    if Self[Result].PlayerID = PlayerID then Break;
    Dec(Result);
  end;
end;

function TPlayerList.Force(PlayerID: string): Integer;
begin
  Result := Find(PlayerID);
  if Result = -1 then begin
    Result := Count;
    Add(TPlayer.Create);
    Self[Result].PlayerID := PlayerID;
  end;
end;

function TPlayerList.FindPack(PackID: string): Integer;
begin
  Result := Count - 1;
  while Result > -1 do begin
    if Self[Result].Pack = PackID then Break;
    Dec(Result);
  end;
end;

{ TGame }

constructor TGame.Create;
begin
  FTeams := TTeamList.Create;
  FPlayers := TPlayerList.Create;
end;

destructor TGame.Destroy;
begin
  FTeams.Free;
  FPlayers.Free;

  inherited;
end;

function TGame.Count: Integer;
begin
  Result := FTeams.Count;
end;

function TGame.TotalScore: Integer;
var i: Integer;
begin
  Result := 0;
  for i := 0 to FTeams.Count - 1 do
    Inc(Result, FTeams[i].Score);
end;

{ TGameList }

procedure TGameList.Put(Index: Integer; Game: TGame);
begin
  inherited Put(Index, Game);
end;

function TGameList.Get(Index: Integer): TGame;
begin
  Result := inherited Get(Index);
end;

function TGameList.Add(Game: TGame): TGame;
begin
  inherited Add(Game);
  Result := Game;
end;

function TGameList.Add: TGame;
begin
  Result := Add(TGame.Create);
end;

procedure TGameList.Flush;
var i: Integer;
begin
  for i := 0 to Count - 1 do
    if Assigned(Self[i]) then begin
      Self[i].Teams.Clear;
      Self[i].Players.Clear;
      Self[i].Free;
      Self[i] := nil;
    end;

  inherited;
end;

function TGameList.Find(GameTime: TDateTime): Integer;
begin
  Result := Count - 1;
  while Result > -1 do begin
    if Abs(Self[Result].GameTime - GameTime) < 0.00035 then Break;  // if they're within 30 seconds or so
    Dec(Result);
  end;
end;

function TGameList.Force(GameTime: TDateTime): Integer;
begin
  Result := Find(GameTime);
  if Result = -1 then begin
    Result := Count;
    Add(TGame.Create);
    Self[Result].GameTime := GameTime;
  end;
end;

function GameCompare(Item1, Item2: Pointer): Integer;
begin
  try
    Result := Round((TGame(Item1).GameTime - TGame(Item2).GameTime) * 21600);  // * 21600 gives us to the nearest 4 seconds, which means an Integer has enough size to carry us from 1899 to 2140 AD or so.
  except
    Result := Integer(Item1) - Integer(Item2);
  end;
end;

procedure TGameList.Sort;
begin
  Sort(GameCompare);
end;

procedure TServerGameList.Put(Index: Integer; ServerGame: TServerGame);
begin
  inherited Put(Index, ServerGame);
end;

function TServerGameList.Get(Index: Integer): TServerGame;
begin
  Result := inherited Get(Index);
end;

function TServerGameList.Add(ServerGame: TServerGame): TServerGame;
begin
  inherited Add(ServerGame);
  Result := ServerGame;
end;

function TServerGameList.Add: TServerGame;
begin
  Result := Add(TServerGame.Create);
end;

procedure TServerGameList.Clear;
var i: Integer;
begin
  for i := 0 to Count - 1 do
    if Assigned(Self[i]) then begin
      Self[i].Free;
      Self[i] := nil;
    end;

  inherited;
end;

function ServerGameCompare(Item1, Item2: Pointer): Integer;
begin
  Result := Round((TServerGame(Item2).GameTime - TServerGame(Item1).GameTime) * 86400);
end;

procedure TServerGameList.Sort;
begin
  Sort(ServerGameCompare);
end;

function TServerGameList.Find(time: TDateTime): TServerGame;
var i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
    if (time > Self[i].GameTime - 1/86400) and (time < Self[i].GameTime + 1/86400) then
      Result := Self[i];
end;

function TServerGameList.Find(gameid: Integer): TServerGame;
var i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
    if gameid = Self[i].Game_ID then
      Result := Self[i];
end;

end.
