unit League;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, IniFiles, ImgList, Grids, DBGrids, Db,
  DBTables, Spin, Math, Menus, ElTree, Buttons;

const
// ColumnText subitems:
    ctHandicap = 0;
    ctButtonID = 1;
    ctComment  = 2;

type
    TLeague = record
        TeamName: string;
        TeamID:   integer;
        Handicap,
        Comment:  string;
        Players,
        ButtonIDs,
        Handicaps,
        Comments: TStringList;
    end;
    PLeague = ^TLeague;

type
  TLeagues = class(TForm)
    TreeView1: TElTree;
    ImageList1: TImageList;
    Splitter1: TSplitter;
    LeagueGroupBox: TGroupBox;
    CopyTeamsFrom: TBitBtn;
    TeamGroupBox: TGroupBox;
    AddTeam: TButton;
    DeleteTeam: TButton;
    RenameTeam: TButton;
    PlayerGroupBox: TGroupBox;
    AddPlayer: TButton;
    DeletePlayer: TButton;
    LeagueStuff: TGroupBox;
    Panel1: TPanel;
    Panel2: TPanel;
    TeamStats: TGroupBox;
    Label11: TLabel;
    TeamHandicap: TEdit;
    Label12: TLabel;
    TeamComment: TEdit;
    TeamStatsGrid: TDBGrid;
    TeamTotalScore: TLabel;
    TeamTotalPoints: TLabel;
    TeamGameCount: TLabel;
    PlayerStats: TGroupBox;
    Label9: TLabel;
    PlayerHandicap: TEdit;
    Label10: TLabel;
    PlayerComment: TEdit;
    PlayerStatsGrid: TDBGrid;
    PlayerAverageScore: TLabel;
    PlayerAverageRank: TLabel;
    PlayerGameCount: TLabel;
    OKButton: TButton;
    CancelButton: TButton;
    PopupMenu1: TPopupMenu;
    DeleteItem: TMenuItem;
    OpenDialog1: TOpenDialog;
    PointsPanel: TPanel;
    Label00: TLabel;
    RankOnScore: TRadioButton;
    RankOnPoints: TRadioButton;
    HighScorePanel: TPanel;
    LabelLow: TLabel;
    LabelHigh: TLabel;
    PointsHigh: TSpinEdit;
    ScrollBox1: TScrollBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeView1ItemCollapse(Sender: TObject; Item: TElTreeItem);
    procedure TreeView1ItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure TreeView1AfterSelectionChange(Sender: TObject);
    procedure TreeView1TryEdit(Sender: TObject; Item: TElTreeItem;
      Section: TElHeaderSection; var CellType: TFieldTypes; var CanEdit: Boolean);
    procedure DoTeamStats;
    procedure DoPlayerStats;
    procedure AddTeamClick(Sender: TObject);
    procedure DeleteTeamClick(Sender: TObject);
    procedure RenameTeamClick(Sender: TObject);
    procedure HandicapExit(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure AddPlayerClick(Sender: TObject);
    procedure DeletePlayerClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RankOnClick(Sender: TObject);
    procedure DeleteItemClick(Sender: TObject);
    procedure StatsGridDrawColumnCell(Sender: TObject;
      const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure CommentExit(Sender: TObject);
    procedure CopyTeamsFromClick(Sender: TObject);
    procedure PointsNNChange(Sender: TObject);
  private
    function  CreateNewPointsEdit(Index, ATop: Integer): TSpinEdit;
    function  NthSuffix(i: Integer): string;
  public
    constructor CreateLeague(LL: TList; LP: Pointer; LPHigh: Pointer; Name, HD: string; Owner: TComponent);
  end;

var Leagues: TLeagues;
    LeagueList: TList;
    LeaguePoints: ^TStringList;
    LeaguePointsHigh: ^Integer;
    HandiDefault: string;

implementation

uses AliasDialog, DataModule;

{$R *.DFM}

function StrDef(sl: TStringList; i: Integer): string;
begin
    if sl.Count > i then
        Result := sl[i]
    else
        Result := '';
end;

// Call the inherited Create, and set up league data in the treeview.
constructor TLeagues.CreateLeague(LL: TList; LP: Pointer; LPHigh: Pointer; Name, HD: string; Owner: TComponent);
var i, j: integer;
    leagptr: PLeague;
    TopNode, TeamNode, PlayerNode: TElTreeItem;
    thisedit: TSpinEdit;
begin
    Leagues := Create(Owner);
    LeagueList := LL;
    LeaguePoints := LP;
    HandiDefault := HD;

    DM.LeagueTable.Close;
    DM.LeagueTable.TableName := ChangeFileExt(ExtractFileName(Name), '');
    DM.LeagueTable.Open;
    TeamStatsGrid.DataSource := DM.LeagueDataSource;
    PlayerStatsGrid.DataSource := DM.LeagueDataSource;

    Caption := 'TORN -- ' + ChangeFileExt(ExtractFileName(Name), '');
    TreeView1.Items.Clear;
    TopNode := TreeView1.Items.Add(nil, ChangeFileExt(ExtractFileName(Name), ''));
    TopNode.ImageIndex := 0;

    for i := 0 to LeagueList.Count - 1 do begin
        leagptr := LeagueList.Items[i];

        if leagptr^.TeamName = '' then begin
            if leagptr^.Players.Count = 0 then
                TeamNode := TreeView1.Items.AddChild(TopNode, 'Team ' + IntToStr(leagptr^.TeamID))
            else
                TeamNode := TreeView1.Items.AddChild(TopNode, leagptr^.Players.Strings[0] + '''s team');
        end else
            TeamNode := TreeView1.Items.AddChild(TopNode, leagptr^.TeamName);
        TeamNode.ImageIndex := 1;
        TeamNode.AllowEdit := True;
        TeamNode.Data := leagptr;
        TeamNode.ColumnText.Add(leagptr^.Handicap);
        TeamNode.ColumnText.Add('');  // ButtonID
        TeamNode.ColumnText.Add(leagptr^.Comment);

        for j := 0 to leagptr^.Players.Count - 1 do begin
            PlayerNode := TreeView1.Items.AddChild(TeamNode, leagptr^.Players.Strings[j]);
            PlayerNode.ImageIndex := 3;
            PlayerNode.Data := leagptr;
            PlayerNode.ColumnText.Add(StrDef(leagptr^.Handicaps, j));
            PlayerNode.ColumnText.Add(StrDef(leagptr^.ButtonIDs, j));
            PlayerNode.ColumnText.Add(StrDef(leagptr^.Comments,  j));
        end;
    end;

    TopNode.Expanded := True;

    thisedit := nil;
    for i := 0 to Max(LeaguePoints^.Count - 1, 2) do begin
        thisedit := CreateNewPointsEdit(i + 1, i * 24 + 54);
        if i < LeaguePoints^.Count then begin
            thisedit.Value := StrToIntDef(LeaguePoints^.Strings[i], 0);
            if LeaguePoints^.Strings[i] <> '0' then RankOnPoints.Checked := True;
        end;
    end;
    thisedit.OnChange := PointsNNChange;

    LeaguePointsHigh := LPHigh;
    PointsHigh.Value := LeaguePointsHigh^;

    if (PointsHigh.Value <> 0) then RankOnPoints.Checked := True;
    RankOnClick(RankOnScore);
end;

procedure TLeagues.FormCreate(Sender: TObject);
begin
    Top  := Round(Screen.Height * 0.10);
    Left := (Screen.Width - Width - 247) div 2;
    if Left < 0 then Left := 0;
end;

procedure TLeagues.FormShow(Sender: TObject);
begin
    if TreeView1.Items.Count > 0 then begin
        TreeView1.Selected := TreeView1.Items.Item[0];
        TreeView1.Items.Item[0].Selected := True;
    end;
end;

// If it's a folder icon, close it.
procedure TLeagues.TreeView1ItemCollapse(Sender: TObject;
  Item: TElTreeItem);
begin
    if Item.ImageIndex = 2 then Item.ImageIndex := 1;
end;

// If it's a folder icon, open it.
procedure TLeagues.TreeView1ItemExpand(Sender: TObject; Item: TElTreeItem);
begin
    if Item.ImageIndex = 1 then Item.ImageIndex := 2;
end;

// A new node has been selected.  Enable and disable buttons accordingly.
procedure TLeagues.TreeView1AfterSelectionChange(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    // States:
    //   0:   League header icon
    //   1,2: Team folder icons (closed, open)
    //   3:   Player laser icon

    with TreeView1.Selected do begin
        AddTeam.Enabled     := ImageIndex in [0,1,2];
        DeleteTeam.Enabled  := ImageIndex in [1,2];
        RenameTeam.Enabled  := ImageIndex in [1,2];

        AddPlayer.Enabled   := ImageIndex in [1,2,3];
        DeletePlayer.Enabled:= ImageIndex in [3];

        LeagueStuff.Visible := ImageIndex in [0];
        TeamStats.Visible   := ImageIndex in [1,2];
        PlayerStats.Visible := ImageIndex in [3];

        if ImageIndex in [1,2] then DoTeamStats;
        if ImageIndex in [3]   then DoPlayerStats;
    end;
end;

// Disallow editing of Players by just single-clicking twice on the field.
procedure TLeagues.TreeView1TryEdit(Sender: TObject; Item: TElTreeItem;
  Section: TElHeaderSection; var CellType: TFieldTypes; var CanEdit: Boolean);
begin
    CanEdit := Item.ImageIndex in [1,2];
end;

// Show statistics for this team in StatsGrid.
procedure TLeagues.DoTeamStats;
var Count, Score, Points: integer;
begin
    if TreeView1.Selected.Data = nil then Exit;

    TeamHandicap.Text := TreeView1.Selected.ColumnText[ctHandicap];
    TeamComment.Text  := TreeView1.Selected.ColumnText[ctComment];

    TeamStats.Caption := 'Stats for ' + PLeague(TreeView1.Selected.Data).TeamName;

    DM.LeagueTable.Filter := 'ID = ' + Format('%4.4d', [PLeague(TreeView1.Selected.Data).TeamID]);
    DM.LeagueTable.Filtered := True;
    DM.LeagueTable.First;
    Count := 0;
    Score := 0;
    Points := 0;
    while not DM.LeagueTable.EOF do begin
        Inc(Count);
        Score := Score + DM.LeagueTable.FieldByName('Score').AsInteger;
        Points := Points + DM.LeagueTable.FieldByName('Points').AsInteger;
        DM.LeagueTable.Next;
    end;

    TeamGameCount.Caption   := 'Total ' + IntToStr(Count) + ' games';
    TeamTotalScore.Caption  := IntToStr(Score);
    TeamTotalPoints.Caption := IntToStr(Points);
end;

// Show statistics for this player in StatsGrid.
procedure TLeagues.DoPlayerStats;
var Count, Score, Points: integer;
begin
    PlayerStats.Caption := 'Stats for ' + TreeView1.Selected.Text;

    PlayerHandicap.Text := TreeView1.Selected.ColumnText[ctHandicap];
    PlayerComment.Text  := TreeView1.Selected.ColumnText[ctComment];

    DM.LeagueTable.Filter := 'ID = ''' + TreeView1.Selected.ColumnText[ctButtonID] + '''';
    DM.LeagueTable.Filtered := True;
    DM.LeagueTable.First;
    Count := 0;
    Score := 0;
    Points := 0;
    while not DM.LeagueTable.EOF do begin
        Inc(Count);
        Score := Score + DM.LeagueTable.FieldByName('Score').AsInteger;
        Points := Points + DM.LeagueTable.FieldByName('Points').AsInteger;
        DM.LeagueTable.Next;
    end;

    PlayerGameCount.Caption := 'Total ' + IntToStr(Count) + ' games';
    if Count = 0 then begin
        PlayerAverageScore.Caption := '';
        PlayerAverageRank.Caption  := '';
    end else begin
        PlayerAverageScore.Caption := IntToStr(Score div Count);
        PlayerAverageRank.Caption  := Format('%f', [Points / Count]);
    end;
end;

procedure TLeagues.AddTeamClick(Sender: TObject);
var TeamNode: TElTreeItem;
begin
    TreeView1.Items[0].Expanded := True;

    TeamNode := TreeView1.Items.AddChild(TreeView1.Items[0], 'New team');
    TeamNode.ImageIndex := 1;
    TeamNode.AllowEdit := True;
    TeamNode.ColumnText.Add('');  // Handicap
    TeamNode.ColumnText.Add('');  // ButtonID
    TeamNode.ColumnText.Add('');  // Comment

    TreeView1.Selected := TeamNode;
    TreeView1.SetFocus;
    TreeView1AfterSelectionChange(Sender);
    TreeView1.Selected.EditText;

    TeamHandicap.Text := '';
    TeamComment.Text  := '';
end;

procedure TLeagues.DeleteTeamClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.Selected.Delete;
end;

procedure TLeagues.RenameTeamClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.SetFocus;
    TreeView1.Selected.EditText;
end;

// Check as focus leaves a Handicap field.  Make sure there is a '+', '-' or '%',
// and remove leading zeroes.
procedure TLeagues.HandicapExit(Sender: TObject);
var s: String;
    val:     Integer;
    plus:    Boolean;
    minus:   Boolean;
    percent: Boolean;
begin
    with Sender as TEdit do begin
        s := Text;
        plus    := (Length(s) > 0) and (s[1] = '+');
        minus   := (Length(s) > 0) and (s[1] = '-');
        percent := (Length(s) > 0) and (s[Length(s)] = '%');

        if plus or minus then s:= Copy(s, 2, 7);

        if Length(s) = 0 then val := 0
        else if percent then  val := StrToIntDef(Copy(s, 1, Length(s)-1), 100)
        else                  val := StrToIntDef(s, 0);

        if percent then                 Text := IntToStr(val) + '%'
        else if val = 0 then            Text := '0'
        else if minus then              Text := '-' + IntToStr(val)
        else if plus then               Text := '+' + IntToStr(val)
        else if HandiDefault = '+' then Text := '+' + IntToStr(val)
        else if HandiDefault = '-' then Text := '-' + IntToStr(val)
        else { must be '%' }            Text := IntToStr(val) + '%';

        TreeView1.Selected.ColumnText[ctHandicap] := Text;
    end;  // with Sender
end;

procedure TLeagues.CommentExit(Sender: TObject);
begin
    TreeView1.Selected.ColumnText[ctComment] := (Sender as TEdit).Text;
end;

// Cause a call to HandicapExit when the user presses Enter.
procedure TLeagues.EditKeyPress(Sender: TObject; var Key: Char);
begin
    if Key = #13 then begin
        TreeView1.SetFocus;
        Key := #0;
    end;
end;

procedure TLeagues.AddPlayerClick(Sender: TObject);
var TeamNode, PlayerNode: TElTreeItem;
    Alias: TAlias;
begin
    TeamNode := TreeView1.Selected;
    while not (TeamNode = nil) and not (TeamNode.ImageIndex in [1,2]) do
        TeamNode := TeamNode.Parent;  // make sure we're at a Team node
    if TeamNode = nil then Exit;

    Alias := TAlias.Create(Application);
    try
        Alias.PlayerAlias.Text := '';
        Alias.Left := Self.Left + Self.Width - Alias.Width - 11;
        Alias.Top  := Self.Top;
        Alias.Height := Self.Height;
        if Alias.ShowModal = mrOK then begin
            PlayerNode := TreeView1.Items.AddChild(TeamNode, Alias.PlayerAlias.Text);
            PlayerNode.ImageIndex := 3;
            PlayerNode.ColumnText.Add('');  // Handicap
            PlayerNode.ColumnText.Add(Alias.ButtonID.Text);  // ButtonID
            PlayerNode.ColumnText.Add('');  // Comment

            TeamNode.Expanded := True;
            TreeView1.Selected := PlayerNode;

            PlayerHandicap.Text := '';
            PlayerComment.Text  := '';
        end;
    finally
        Alias.Release;
    end;
end;

procedure TLeagues.DeletePlayerClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.Selected.Delete;
    TreeView1.SetFocus;
end;

// The user has clicked the OK button, to save his changes and close the form.
// Recreate the LeagueList based on the information in the TreeView.
// (Form closing is handled automagically because OKButton's ModalResult is set.)
procedure TLeagues.OKButtonClick(Sender: TObject);
var i: Integer;
    leagptr: PLeague;
    League2: TList;
    highteamID: integer;
begin
    highteamID := 1;
    for i := 0 to Leagues.TreeView1.Items.Count - 1 do  // Find the highest teamID, and set highteamID one higher.
        if (TreeView1.Items[i].ImageIndex in [1,2]) and  // this is a team title
           (TreeView1.Items[i].Data <> nil) and          // this team has an allocated teamID
           (highteamID <= PLeague(TreeView1.Items[i].Data).TeamID) then
            highteamID := PLeague(TreeView1.Items[i].Data).TeamID + 1;

    League2 := TList.Create;
    leagptr := nil;

    for i := 0 to TreeView1.Items.Count - 1 do
        with TreeView1.Items[i] do begin
            if ImageIndex in [1,2] then begin  // this is a team title,
                New(leagptr);                  // so set up an entry for this team in the List.
                League2.Add(leagptr);
                leagptr^.TeamName := Text;
                if Data = nil then begin
                    leagptr^.TeamID := highteamID;
                    Inc(highteamID);
                end else
                    leagptr^.TeamID := PLeague(Data).TeamID;
                leagptr^.Handicap := ColumnText[ctHandicap];
                leagptr^.Comment  := ColumnText[ctComment];

                leagptr^.Players  := TStringList.Create;
                leagptr^.ButtonIDs:= TStringList.Create;
                leagptr^.Handicaps:= TStringList.Create;
                leagptr^.Comments := TStringList.Create;
            end else if ImageIndex = 3 then begin  // this is a player
                leagptr^.Players.Add(Text);
                leagptr^.ButtonIDs.Add(ColumnText[ctButtonID]);
                leagptr^.Handicaps.Add(ColumnText[ctHandicap]);
                leagptr^.Comments.Add(ColumnText[ctComment]);
            end;
        end;  // with TreeView1, for i

    for i := LeagueList.Count - 1 downto 0 do begin  // Now throw away the old list,
        Dispose(LeagueList.Items[i]);
        LeagueList.Delete(i);
    end;

    for i := 0 to League2.Count - 1 do      // copy in records fom the new one,
        LeagueList.Add(League2[i]);

    for i := League2.Count - 1 downto 0 do  // and throw away originals.
        League2.Delete(i);

    LeaguePoints^.Clear;
    for i := 0 to ComponentCount - 1 do
        if (Components[i].Tag <> 0) and (Components[i] is TSpinEdit) then begin
            LeaguePoints^.Add(IntToStr((Components[i] as TSpinEdit).Value));  // This code assumes that the PointsNN spinedits are in the Components array in increasing order.
            if (Components[i] as TSpinEdit).Value = 0 then Break;
        end;
    LeaguePointsHigh^ := PointsHigh.Value;
end;

procedure TLeagues.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    try
        DM.LeagueTable.Filter := '';
        DM.LeagueTable.Filtered := False;
        DM.LeagueTable.Close;
    finally
        ;
    end;
end;

// Enable / disable controls
procedure TLeagues.RankOnClick(Sender: TObject);
var i: Integer;
begin
    for i := 0 to ComponentCount - 1 do
        if Components[i].Tag <> 0 then
            (Components[i] as TControl).Enabled := RankOnPoints.Checked;

    LabelLow.Enabled   := RankOnPoints.Checked;
    LabelHigh.Enabled  := RankOnPoints.Checked;
    PointsHigh.Enabled := RankOnPoints.Checked;

    if RankOnScore.Checked then begin
    for i := 0 to ComponentCount - 1 do
        if (Components[i].Tag <> 0) and (Components[i] is TSpinEdit) then
            (Components[i] as TSpinEdit).Value := 0;
        PointsHigh.Value := 0;
    end;
end;

procedure TLeagues.DeleteItemClick(Sender: TObject);
begin
    if not DM.LeagueTable.EOF then DM.LeagueTable.Delete;
end;

// Colour background to match colour of pack or team.
// Shorten date/time displayed in Game field so it (hopefully) fits.
// Show team names in the "ID2" (team) column, instead of team numbers.
procedure TLeagues.StatsGridDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
var i:        Integer;
    teamnum:  Integer;
    teamname: string;
    s: string;
begin
    if (State = []) then begin
        s := UpperCase(Copy(DM.LeagueTable.FieldByName('Pack').AsString, 1, 2));

        with (Sender as TDBGrid).Canvas.Brush do begin
            if s = 'RE' then Color := $02E0E0FF;
            if s = 'GR' then Color := $02E0FFD8;
            if s = 'BL' then Color := $02FFE8D0;
            if s = 'YE' then Color := $02D0FFFF;
        end;
    end;

    if (Column.FieldName = 'Game') and (State = []) then begin
        s := Column.Field.AsString;
        if (Sender as TDBGrid).Canvas.TextWidth(s) < Rect.Right - Rect.Left then
            (Sender as TDBGrid).DefaultDrawColumnCell(Rect, DataCol, Column, State)
        else begin
            s := StringReplace(ShortDateFormat, DateSeparator + 'yy', '', []);
            s := StringReplace(s, 'yy' + DateSeparator, '', []);
            s := StringReplace(s, 'yy', '', []);
            s := FormatDateTime(s + ' ' + ShortTimeFormat, Column.Field.AsDateTime);
            (Sender as TDBGrid).Canvas.TextRect(Rect, Rect.Left + 3, Rect.Top + 2, s);
        end;
    end else if (Sender = PlayerStatsGrid) and (Column.FieldName = 'ID2') and (State = []) then begin
        teamname := Column.Field.AsString;
        teamnum  := StrToIntDef(teamname, 0);
        for i := 0 to Leagues.TreeView1.Items.Count - 1 do
            if (TreeView1.Items[i].Data <> nil) and
               (PLeague(TreeView1.Items[i].Data).TeamID = teamnum) then begin
                teamname := TreeView1.Items[i].Text;
                Break;
            end;

        (Sender as TDBGrid).Canvas.TextRect(Rect, Rect.Left + 3, Rect.Top + 2, teamname);
    end else
        (Sender as TDBGrid).DefaultDrawColumnCell(Rect, DataCol, Column, State);
end;

procedure TLeagues.CopyTeamsFromClick(Sender: TObject);
var MyIni:  TIniFile;
    TeamNode, PlayerNode: TElTreeItem;
    Players, ButtonIDs, Handicaps, Comments: TStringList;
    i, j:   Integer;
    s, p:   string;
begin
    if not OpenDialog1.Execute then Exit;

    MyIni := TIniFile.Create(OpenDialog1.FileName);
    TreeView1.Items[0].Expanded := True;
    Players   := TStringList.Create;
    ButtonIDs := TStringList.Create;
    Handicaps := TStringList.Create;
    Comments  := TStringList.Create;

    i := 0;
    while True do begin
        s := 'T' + Format('%2.2d', [i+1]);  // set up the ini section name
        if MyIni.ReadString(s, 'TeamName', 'no-match') = 'no-match' then break;

        TeamNode := TreeView1.Items.AddChild(TreeView1.Items[0], 'New team');
        TeamNode.Text := MyIni.ReadString(s, 'TeamName', '');
        TeamNode.ImageIndex := 1;
        TeamNode.AllowEdit := True;
        TeamNode.ColumnText.Add(MyIni.ReadString(s, 'Handicap', '0'));
        TeamNode.ColumnText.Add('');  // ButtonID
        TeamNode.ColumnText.Add(MyIni.ReadString(s, 'Comment', ''));

        p := MyIni.ReadString(s, 'Players', '');
        if Pos(' ', p) < Pos('"', p) then p := '"' + p + '"';  // Note: ReadString is evil.  See TornMain.pas for details.
        Players.CommaText := p;

        ButtonIDs.CommaText := MyIni.ReadString(s, 'ButtonIDs', '');
        Handicaps.CommaText := MyIni.ReadString(s, 'Handicaps', '');

        p := MyIni.ReadString(s, 'Comments', '');
        if Pos(' ', p) < Pos('"', p) then p := '"' + p + '"';  // Note: ReadString is evil.  See TornMain.pas for details.
        Comments.CommaText  := p;

        for j := 0 to Min(Players.Count, ButtonIDs.Count) - 1 do begin
            PlayerNode := TreeView1.Items.AddChild(TeamNode, Players[j]);
            PlayerNode.ImageIndex := 3;
            PlayerNode.ColumnText.Add(StrDef(Handicaps, j));
            PlayerNode.ColumnText.Add(StrDef(ButtonIDs, j));
            PlayerNode.ColumnText.Add(StrDef(Comments,  j));
        end;

        Inc(i);
    end;  // for i

    MyIni.Free;
    Players.Free;
    ButtonIDs.Free;
    Handicaps.Free;
    Comments.Free;
end;

// When the value of this SpinEdit first becomes non-zero,
// add a new SpinEdit below it, and move this code to the new SpinEdit.
procedure TLeagues.PointsNNChange(Sender: TObject);
begin
    if (Sender as TSpinEdit).Value <> 0 then begin

        LabelLow.Caption := 'Points for ' + IntToStr((Sender as TSpinEdit).Tag + 1) + 'th and lower: 0';

        CreateNewPointsEdit((Sender as TSpinEdit).Tag + 1, (Sender as TSpinEdit).Top + 24).OnChange := PointsNNChange;

        (Sender as TSpinEdit).OnChange := nil;
    end;
end;

// Add a new SpinEdit and accompanying label
function TLeagues.CreateNewPointsEdit(Index, ATop: Integer): TSpinEdit;
begin
    with TLabel.Create(Self) do begin
        Parent  := Label00.Parent;
        Left    := 40;
        Top     := ATop + 3;
        Height  := 13;
        Tag     := Index;
        Caption := 'Points for ' + NthSuffix(Index);
    end;  // with

    Result := TSpinEdit.Create(Self);
    with Result do begin
        Parent  := Label00.Parent;
        Left    := 112;
        Top     := ATop;
        Width   := 41;
        Height  := 22;
        MinValue:= 0;
        MaxValue:= 99;
        Tag     := Index;
    end;  // with
end;

function TLeagues.NthSuffix(i: Integer): string;
begin
    case i mod 10 of
        1:   Result := 'st';
        2:   Result := 'nd';
        3:   Result := 'rd';
        else Result := 'th';
    end;
    if i in [11, 12, 13] then Result := 'th';
    Result := IntToStr(i) + Result;
end;

end.
