unit PyramidDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, ExtCtrls, ComCtrls, Games, League, ZoomReport,
  Menus;

const
  COL_TITLE = 0;
  COL_NUM_TEAMS = 1;
  COL_TEAMS_TO_TAKE = 2;
  COL_SECRET = 3;

type
  TPyramidForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    TeamsPerGame: TLabel;
    Games: TSpinEdit;
    TeamsFromRound: TSpinEdit;
    GamesView: TListView;
    TopOrBottom: TRadioGroup;
    TeamsToTakeLabel: TLabel;
    SendTo: TPanel;
    Label7: TLabel;
    ToScreen: TCheckBox;
    ToScoreboard: TCheckBox;
    ToPrinter: TCheckBox;
    Portrait: TRadioButton;
    Landscape: TRadioButton;
    ToHtml: TCheckBox;
    ToTsv: TCheckBox;
    CancelButton: TButton;
    OKButton: TButton;
    RefreshButton: TButton;
    ScoreOrRank: TRadioGroup;
    Label5: TLabel;
    RoundTitle: TEdit;
    TeamsFromRep: TSpinEdit;
    Label6: TLabel;
    Label8: TLabel;
    TeamsTotal: TLabel;
    GamesPopup: TPopupMenu;
    RoundMenu: TMenuItem;
    RepechageMenu: TMenuItem;
    RoundNMenu: TMenuItem;
    RepechageNMenu: TMenuItem;
    ClearMenu: TMenuItem;
    ClearButton: TButton;
    SendSpin: TSpinEdit;
    Label9: TLabel;
    Label10: TLabel;
    SendButton: TButton;
    RefreshTimer: TTimer;
    TitleMenu: TMenuItem;
    Label11: TLabel;
    ColourButton: TButton;
    PanelMiddle: TPanel;
    Preview: TImage;
    Past: TImage;
    Splitter1: TSplitter;
    PanelPast: TPanel;
    PanelDraw: TPanel;
    SecretMenu: TMenuItem;
    procedure SpinChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RefreshButtonClick(Sender: TObject);
    procedure ChooseTeamsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RoundMenuClick(Sender: TObject);
    procedure RoundNMenuClick(Sender: TObject);
    procedure ClearMenuClick(Sender: TObject);
    procedure GamesViewDblClick(Sender: TObject);
    procedure RoundTitleChange(Sender: TObject);
    procedure RefreshTimerTimer(Sender: TObject);
    procedure TitleMenuClick(Sender: TObject);
    procedure ColourButtonClick(Sender: TObject);
    procedure SecretMenuClick(Sender: TObject);
    procedure GamesPopupPopup(Sender: TObject);
  private
    FDraw: TZoomReports;
    FPast: TZoomReports;

    procedure Refresh;
    procedure CalcSpins;
  public
    GameList: TGameList;
    LeagueList: TLeagueList;

    property Reports: TZoomReports read FDraw;
  end;

  TTeamTotal = class
    private
        FTeamID: Integer;
        FTotalScore,
        FTotalRank,
        FCount: Integer;  // Number of games this team has played
        FTotalPoints: Double;
        FGame: TGame;  // If this team has played one game, this is it. If they're played more than one, this is their first.
        FTeam: Integer;  // This is this team's index within that game's team list.
    public
        Constructor Create;

        property TeamID:      Integer read FTeamID      write FTeamID;
        property TotalScore:  Integer read FTotalScore  write FTotalScore;
        property TotalPoints: Double  read FTotalPoints write FTotalPoints;
        property TotalRank:   Integer read FTotalRank   write FTotalRank;
        property Count:       Integer read FCount       write FCount;
        property Game:        TGame   read FGame        write FGame;
        property Team:        Integer read FTeam        write FTeam;
    end;

var
  PyramidForm: TPyramidForm;
  RankedList: TList;  // of TTeamTotal
  TopFirst: Boolean;
  
implementation

uses Math, Lists, Utility;

{$R *.dfm}

function LastGame(Row: TZRow; Columns: TZColumns = nil): Integer;
begin
  Result := Row.Count - 1;

  if Assigned(Columns) then
    while (Result > 0) and ((Columns[Result].Heading = 'Dropped') or (Columns[Result].Heading = 'Average score') or (Columns[Result].Heading = 'Pts')) do
      Dec(Result);

  while (Result > 0) and (Row[Result].Text = '') do
    Dec(Result);
end;

function RankedListPos(teamid: Integer): Integer;
begin
  Result := RankedList.Count - 1;

  while (Result > -1) and (TTeamTotal(RankedList[Result]).TeamID <> teamid) do
    Dec(Result);
end;

function CompareRank(Item1, Item2: Pointer): Integer;
var rank1, rank2: Integer;
begin
  rank1 := RankedListPos(StrToInt(TZRow(Item1)[1].Text));
  rank2 := RankedListPos(StrToInt(TZRow(Item2)[1].Text));
  if (rank1 > -1) and (rank2 > -1) then Result := rank1 - rank2
  else if (rank1 > -1) and TopFirst then Result := -1
  else if (rank2 > -1) and TopFirst then Result := +1
  else if rank1 > -1 then Result := +1
  else if rank2 > -1 then Result := -1
  else
    Result := LastGame(Item2) - LastGame(Item1);  // Rank higher the team with the more recent game.
  if Result = 0 then
    Result := StrToIntDef(TZRow(Item2)[LastGame(Item2)].Text, 0) - StrToIntDef(TZRow(Item1)[LastGame(Item1)].Text, 0);  // Compare points in each team's most recent game.
  if Result = 0 then
    Result := StrToIntDef(TZRow(Item2)[LastGame(Item2) - 1].Text, 0) - StrToIntDef(TZRow(Item1)[LastGame(Item1) - 1].Text, 0);  // Compare scores in each team's most recent game.
end;

procedure TPyramidForm.FormDestroy(Sender: TObject);
begin
  FDraw.Free;
  FPast.Free;
end;

procedure SetPosition(control: TControl; rightspace: Integer = 0; bottomspace: Integer = 0);
begin
  if akTop in control.Anchors then
    control.Height := Min(control.Height, control.Parent.Height - control.Top - bottomspace);

  if akBottom in control.Anchors then
    control.Top := Min(control.Top, control.Parent.Height - control.Height - 4 - bottomspace);

  if akLeft in control.Anchors then
    control.Width := Min(control.Width, control.Parent.Width - control.Left - rightspace);

  if akRight in control.Anchors then
    control.Left := Min(control.Left, control.Parent.Width - control.Width - 4 - rightspace);
end;

procedure TPyramidForm.FormShow(Sender: TObject);
var
  i: Integer;
  item: TListItem;
begin
  Top := 1;
  Height := Round(Screen.Height * 0.9);

  if GamesView.Items.Count < GameList.Count then
    for i := GamesView.Items.Count to GameList.Count - 1 do begin
      item := GamesView.Items.Add;

      item.Caption := FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[i].GameTime);
      item.SubItems.Add(GameList[i].Title);  // label
      item.SubItems.Add(IntToStr(GameList[i].Teams.Count));  // num teams
      item.SubItems.Add('');  // num teams to take
      if GameList[i].Secret then
        item.SubItems.Add('y')  // secret?
      else
        item.SubItems.Add('');  // secret?
    end;
end;

procedure TPyramidForm.ChooseTeamsClick(Sender: TObject);
begin
//  ScoreOrRank.Enabled := ChooseTeams.ItemIndex in [0, 1];
//  TeamsToTakeLabel.Enabled := ChooseTeams.ItemIndex = 1;

  SpinChange(Sender);
end;

procedure TPyramidForm.SpinChange(Sender: TObject);
begin
  TeamsTotal.Caption := FloatToStr(TeamsFromRound.Value + TeamsFromRep.Value);
  TeamsPerGame.Caption := FloatToStrF((TeamsFromRound.Value + TeamsFromRep.Value) / Games.Value, ffNumber, 6, 3);
  Refresh;
end;

procedure TPyramidForm.RoundMenuClick(Sender: TObject);
var i: Integer;
begin
  for i := 0 to GamesView.Items.Count - 1 do
    if GamesView.Items[i].Selected then
      GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE] := TMenuItem(Sender).Caption;

  Refresh;
  CalcSpins;
end;

procedure TPyramidForm.RoundNMenuClick(Sender: TObject);
var
  r: string;
  s: string;
  i: Integer;
begin
  if GamesView.SelCount > 0 then begin
    if Sender = RoundNMenu then
      r := 'Round'
    else
      r := 'Repechage';

    i := Pos(', ', GamesView.Selected.SubItems[COL_TEAMS_TO_TAKE]);
    if i > 0 then
      s := Trim(Copy(GamesView.Selected.SubItems[COL_TEAMS_TO_TAKE], i + 1, MaxInt))
    else
      s := '0';

    if ((GamesView.SelCount = 1) and
        InputQuery('', 'Number of teams to bring from ' + r + ' game ' + GamesView.Selected.Caption + ':', s)) or
       InputQuery('', 'Number of teams to bring from each of ' + IntToStr(GamesView.SelCount) + ' ' + r + ' games:', s) then begin
      for i := 0 to GamesView.Items.Count - 1 do
        if GamesView.Items[i].Selected then begin
          if StrToIntDef(s, 0) > StrToIntDef(GamesView.Items[i].Subitems[COL_NUM_TEAMS], 0) then
            GamesView.Items[i].Subitems[COL_TEAMS_TO_TAKE] := r + ', ' + GamesView.Items[i].Subitems[COL_NUM_TEAMS]
          else
            GamesView.Items[i].Subitems[COL_TEAMS_TO_TAKE] := r + ', ' + s;
        end;
      Refresh;
    end;
  end;

  CalcSpins;
end;

procedure TPyramidForm.TitleMenuClick(Sender: TObject);
var
  s: string;
  b: Boolean;
  i: Integer;
begin
  if GamesView.SelCount > 0 then begin
    s := GamesView.Selected.SubItems[COL_TITLE];

    if GamesView.SelCount = 1 then
      b := InputQuery('', 'Label game ' + GamesView.Selected.Caption + ':', s)
    else
      b := InputQuery('', 'Label each of ' + IntToStr(GamesView.SelCount) + ' games:', s);

    if b then begin
      for i := 0 to GamesView.Items.Count - 1 do
        if GamesView.Items[i].Selected then begin
          GamesView.Items[i].Subitems[COL_TITLE] := s;
          GameList[i].Title := s;  // Write this back into the MainForm's GameList.
        end;
      Refresh;
    end;
  end;
end;

procedure TPyramidForm.GamesViewDblClick(Sender: TObject);
begin
  RoundNMenuClick(RoundNMenu);
end;

procedure TPyramidForm.CalcSpins;
var
  i, j: Integer;
  roundteams: Integer;
  repteams: Integer;
begin
  roundteams := 0;
  repteams := 0;
  for i := 0 to GameList.Count - 1 do begin  // for each game
    j := Pos(', ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]);
      if j > 0 then begin
        j := StrToIntDef(Copy(GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE], j + 1, MaxInt), 0);  // j is number of teams to take from this game.
        if Pos('Round, ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
          Inc(roundteams, j)
        else if Pos('Repechage, ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
          Inc(repteams, j);
      end else
        if Pos('Round', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
          Inc(roundteams, StrToIntDef(GamesView.Items[i].SubItems[COL_NUM_TEAMS], 0))
        else if Pos('Repechage', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
          Inc(repteams, StrToIntDef(GamesView.Items[i].SubItems[COL_NUM_TEAMS], 0));
  end;
  TeamsFromRound.Value := roundteams;
  TeamsFromRep.Value := repteams;
end;

procedure TPyramidForm.ClearMenuClick(Sender: TObject);
var i: Integer;
begin
  for i := 0 to GamesView.Items.Count - 1 do
    if GamesView.Items[i].Selected then begin
      GamesView.Items[i].Subitems[COL_TEAMS_TO_TAKE] := '';
      Refresh;
    end;

  CalcSpins;
end;

procedure TPyramidForm.RefreshButtonClick(Sender: TObject);
begin
  Refresh;
end;

procedure TPyramidForm.Refresh;

  procedure AddTeamTotal(game: TGame; team: TTeam; total: TList; rank: Integer);
  var
    i : Integer;  // Index of this team in teamtotal[].
    teamtotal: TTeamTotal;
  begin
    i := total.Count - 1;
    while (i >= 0) and (TTeamTotal(total[i]).TeamID <> team.TeamID) do
      Dec(i);

    if i = -1 then begin
      teamtotal := TTeamTotal.Create;
      teamtotal.TeamID := team.TeamID;
      total.Add(teamtotal);
    end else
      teamtotal := TTeamTotal(total[i]);

    teamtotal.TotalScore := teamtotal.TotalScore + team.Score;
    teamtotal.TotalPoints := teamtotal.TotalPoints + team.Points;
    teamtotal.TotalRank := teamtotal.TotalRank + rank + 1;
    teamtotal.Count := teamtotal.Count + 1;
    if not Assigned(teamtotal.Game) then
      teamtotal.Game := game;
  end;

  procedure AddGame(game: TGame; totallist: TList; from, too: Integer);
  var j: Integer;
  begin
    if TopFirst then // take the top N
      for j := from to Min(too, game.Teams.Count) - 1 do
        AddTeamTotal(game, game.Teams[j], totallist, j)
    else  // take the bottom N
      for j := game.Teams.Count - too to game.Teams.Count - Min(from, game.Teams.Count) - 1 do
        AddTeamTotal(game, game.Teams[j], totallist, j);
  end;

  // If ScoreOrRank.ItemIndex = 0 then compare teams on victory points then score then rank.
  // Otherwise, compare on rank, then on victory points then score.
  function Compare(a, b: TTeamTotal) : Integer;
  begin
    if ScoreOrRank.ItemIndex = 1 then
      Result := a.TotalRank * b.Count - b.TotalRank * a.Count
    else
      Result := 0;

    if Result = 0 then
      Result := Sign(b.TotalPoints * a.Count - a.TotalPoints * b.Count);

    if Result = 0 then
      Result := b.TotalScore * a.Count - a.TotalScore * b.Count;

    if Result = 0 then
      Result := a.TotalRank * b.Count - b.TotalRank * a.Count
  end;

  procedure BogoSort(list: TList);
  var
    i, j: Integer;
  begin
    for i := 0 to list.Count - 1 do
      for j := i to list.Count - 1 do
        if Compare(TTeamTotal(list[i]), TTeamTotal(list[j])) > 0 then
          list.Exchange(i, j);
  end;

  procedure GetRest(list: TList; roundorrep: String; numtoget: Integer);
  var
    i, j: Integer;
    taillist: TList;
  begin
    Bogosort(list);

    // Remove entries that are too low/high.
    while list.Count > numtoget do
      if TopFirst then begin  // Remove lowest.
        TTeamTotal(list[list.Count - 1]).Free;
        list.Delete(list.Count - 1);
      end else begin  // Remove highest.
        TTeamTotal(list[0]).Free;
        list.Delete(0);
      end;

    // Add more entries if necessary.
    if list.Count < numtoget then begin
      taillist := TList.Create;
      try
        for i := 0 to GameList.Count - 1 do begin  // for each game
          j := Pos(roundorrep, GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]);
          if j > 0 then begin
            j := StrToIntDef(Copy(GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE], j + Length(roundorrep), MaxInt), 0);  // j is number of teams we already took from this game.
            AddGame(GameList[i], taillist, j, GameList[i].Teams.Count);  // Add all the teams that didn't get brought in before.
          end;
        end;

        BogoSort(taillist);
        if TopFirst then
          for i := 0 to Min(numtoget - list.Count, taillist.Count) - 1 do
            list.Add(taillist[i])
        else
          for i := taillist.Count - Min(numtoget - list.Count, taillist.Count) to taillist.Count - 1 do
            list.Add(taillist[i]);
      finally
//      for i := 0 to taillist.Count - 1 do
//        TTeamTotal(taillist[i]).Free;
        taillist.Free;
      end;
    end;  // if list.Count < num
  end;

  function RowIsEmpty(row: TZRow): Boolean;
  var i: Integer;
  begin
    Result := True;
    for i := 2 to row.Count - 1 do
      if row[i].Text <> '' then Result := False;
  end;

  procedure HighlightPast(report: TZoomReport; teamtotal: TTeamTotal);
  var
    i, j: Integer;
    teamingame: Integer;
    gamecount: Integer;
  begin
    gamecount := MaxInt;
    for i := 0 to report.Rows.Count - 1 do
      if report.Rows[i][1].Text = IntToStr(teamtotal.TeamID) then
         for j := 2 to report.Rows[i].Count - 1 do
           if report.Rows[i][j].Text <> '' then begin
             teamingame := teamtotal.Game.Teams.Find(teamtotal.TeamID);
             if teamtotal.Game.Teams[teamingame].Score = StrToInt(report.Rows[i][j].Text) then begin
               report.Rows[i][1].Color := $0280C0F0;  // Set team name orange.
               report.Rows[i][j].Color := $0280C0F0;  // Set score orange.
               gamecount := 1;
             end else if gamecount < teamtotal.Count then begin
               report.Rows[i][j].Color := $0280C0F0;  // Set subsequent score orange.
               Inc(gamecount);
             end;
           end;
  end;

var
  i, j: Integer;
  roundlist: TList;  // Team totals from previous round. List of TTeamTotal.
  replist: TList;  // Team totals from previous repechage. List of TTeamTotal.
  row: TZRow;
  rownum: Integer;
  leagueteam: TLeagueTeam;
  teamingame: Integer;
begin  //----Refresh----
  // Build a list of all the teams from the previous round followed by all the teams from the previous repechage.
  // Then pull off that list into the report, either from the low end or the high end.

  TopFirst := TopOrBottom.ItemIndex = 0;

  roundlist := TList.Create;
  replist := TList.Create;
  try
    for i := 0 to GameList.Count - 1 do begin  // for each game
      GameList[i].Teams.Sort;
      j := Pos(', ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]);
      if j > 0 then
        j := StrToIntDef(Copy(GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE], j + 1, MaxInt), 0);  // j is number of teams to take from this game.

      if Pos('Round, ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
        AddGame(GameList[i], roundlist, 0, j)  // Add teams to teams-from-previous-round list.
      else if Pos('Repechage, ', GamesView.Items[i].SubItems[COL_TEAMS_TO_TAKE]) > 0 then
        AddGame(GameList[i], replist, 0, j)  // Add teams to teams-from-previous-repechage list.
    end;

    GetRest(roundlist, 'Round', TeamsFromRound.Value);
    GetRest(replist, 'Repechage', TeamsFromRep.Value);

    for i := 0 to replist.Count - 1 do  // Add all the repechage games into the main list.
      roundlist.Add(replist[i]);

    // Build the "past" report: one row per team, one game per column.
    FPast.Free;
    FPast := TZoomReports.Create;
    FPast.Add('From', 'Rank,Team', 'center,left');

    for i := 0 to LeagueList.Count - 1 do begin  // For each League team, create a row.
       FPast[0].Rows.Add(TZRow.Create);
       FPast[0].Rows[i].Add('');  // rank
       FPast[0].Rows[i].Add(IntToStr(LeagueList[i].TeamID));  // Put in team ID temporarily.
    end;

    for i := 0 to GamesView.Items.Count - 1 do
      if GamesView.Items[i].Subitems[COL_TEAMS_TO_TAKE] <> '' then begin
        FPast[0].Columns.Add(GamesView.Items[i].Caption, 'right');
        for j := 0 to FPast[0].Rows.Count - 1 do begin  // For each League team,
          teamingame := GameList[i].Teams.Find(StrToInt(FPast[0].Rows[j][1].Text));
          if teamingame > -1 then
            FPast[0].Rows[j].Add(IntToStr(GameList[i].Teams[teamingame].Score))
          else
            FPast[0].Rows[j].Add('');
        end;
      end;

    for i := FPast[0].Rows.Count - 1 downto 0 do  // For each League team,
      if RowIsEmpty(FPast[0].Rows[i]) then
        FPast[0].Rows.Delete(i);

    // Build the Draw report: one future game per column.
    FDraw.Free;
    FDraw := TZoomReports.Create;
    FDraw.Add(RoundTitle.Text, '', '');
    FDraw[0].MultiColumnOK := False;

    for i := 1 to Games.Value do
      FDraw[0].Columns.Add('Game ' + IntToStr(i), 'left');

    i := 0;
    while i < roundlist.Count do begin
      row := FDraw[0].Rows[FDraw[0].Rows.Add(TZRow.Create)];

      if i + Games.Value <= roundlist.Count then  // This is a complete row.
        for j := 0 to Games.Value - 1 do begin
          rownum := FDraw[0].Rows.Count;
          if rownum mod 2 = 1 then
            leagueteam := LeagueList.FindTeamID(TTeamTotal(roundlist[i]).TeamID)  // Zig.  Run this row forward: team1 in game 1, etc.
          else
            leagueteam := LeagueList.FindTeamID(TTeamTotal(roundlist[rownum * Games.Value - j - 1]).TeamID);  // Zag.  Run this row backwards: first team in last game, etc.
          if Assigned(leagueteam) then begin
            row.Add(leagueteam.TeamName);
            HighlightPast(FPast[0], TTeamTotal(roundlist[i]));
          end else
            row.Add('--');

          Inc(i);
        end
      else  // This is the final, partial row.
        for j := 0 to Games.Value - 1 do begin
          if i < roundlist.Count then begin
            leagueteam := LeagueList.FindTeamID(TTeamTotal(roundlist[i]).TeamID);
            if Assigned(leagueteam) then begin
              row.Add(leagueteam.TeamName);
              HighlightPast(FPast[0], TTeamTotal(roundlist[i]));
            end else
              row.Add('--');
          end else
            row.Add('-');

          Inc(i);
        end;
    end;

    // Finalise the Past report.
    RankedList := roundlist;
    BubbleSort(FPast[0].Rows, CompareRank);
    for i := 0 to FPast[0].Rows.Count - 1 do begin  // For each League team,
      FPast[0].Rows[i][0].Text := IntToStr(i + 1);  // Set rank.
      FPast[0].Rows[i][1].Text := LeagueList.FindTeamID(StrToInt(FPast[0].Rows[i][1].Text)).BuildTeamName;  // Fill in their team name.
    end;

    FormResize(nil);  // Render the report.
  finally
    for i := 0 to roundlist.Count - 1 do
      TTeamTotal(roundlist[i]).Free;
    roundlist.Free;
    replist.Free;
  end;

  SendSpin.MaxValue := Games.Value;
end;

procedure TPyramidForm.ColourButtonClick(Sender: TObject);
const
  ColorOrder: array[0..7] of TColor = (0, 1, 2, 3, 6, 5, 4, 7);
var
  game, spot, i, spot2: Integer;
  tempcolor: TColor;
  report: TZoomReport;
begin
  if Assigned(FDraw) and (FDraw.Count >= 1) then begin
    report := FDraw[0];
    for game := 0 to report.Columns.Count - 1 do begin
      // Set starting colours.
      for spot := 0 to report.Rows.Count - 1 do
        report.Rows[spot][game].Color := NexusColors[ColorOrder[spot mod 8]];

      // Shuffle colours.
      for i := 1 to 100 do begin
        spot := Random(report.Rows.Count);
        spot2 := Random(report.Rows.Count);

        tempcolor := report.Rows[spot][game].Color;
        report.Rows[spot][game].Color := report.Rows[spot2][game].Color;
        report.Rows[spot2][game].Color := tempcolor;
      end;
    end;
    FormResize(nil);  // Render the report.
  end;
end;

procedure TPyramidForm.FormResize(Sender: TObject);
begin
  Preview.Picture.Bitmap.Height := Preview.Height;
  Preview.Picture.Bitmap.Width  := Preview.Width;

  Past.Picture.Bitmap.Height := Past.Height;
  Past.Picture.Bitmap.Width  := Past.Width;

  //  Past.Width := (Panel1.Width - Splitter1.Width) div 2;

  if Assigned(FDraw) then
    FDraw.Show(Preview.Canvas, 1, 1, Preview.Width - 2, Preview.Height - 2);
  if Assigned(FPast) then
    FPast.Show(Past.Canvas, 1, 1, Past.Width - 2, Past.Height - 2);
end;

{ TTeamTotal }

constructor TTeamTotal.Create;
begin
  FTeamID := -1;
  FTeam := -1;
end;

procedure TPyramidForm.RoundTitleChange(Sender: TObject);
begin
  RefreshTimer.Enabled := False;
  RefreshTimer.Interval := 1000;
  RefreshTimer.Enabled := True;
end;

procedure TPyramidForm.RefreshTimerTimer(Sender: TObject);
begin
  RefreshTimer.Enabled := False;
  Refresh;
end;

procedure TPyramidForm.SecretMenuClick(Sender: TObject);
var
  i: Integer;
  b: Boolean;
  s: string;
begin
  b := SecretMenu.Caption = 'Make &Secret';
  if b then
    s := 'y'
  else
    s := '';

  for i := 0 to GamesView.Items.Count - 1 do
    if GamesView.Items[i].Selected then begin
      GameList[i].Secret := b;
      GamesView.Items[i].SubItems[COL_SECRET] := s;
    end;
end;

procedure TPyramidForm.GamesPopupPopup(Sender: TObject);
var
  i: Integer;
  b: Boolean;
begin
  b := False;
  for i := 0 to GamesView.Items.Count - 1 do
    if GamesView.Items[i].Selected then
      b := b or (GamesView.Items[i].SubItems[COL_SECRET] = 'y');

  if b then
    SecretMenu.Caption := 'Clear &Secret'
  else
    SecretMenu.Caption := 'Make &Secret';
end;

end.
