unit RegistryDef;

interface

uses
  Controls, StdCtrls, ExtCtrls, SysUtils, Classes, Registry, Spin;

type
  TRegistryDef = class(TRegistry)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    function ReadBoolDef(Name: String; Default: Boolean): Boolean; virtual;
    function ReadIntegerDef(Name: String; Default: Integer): Integer; virtual;
    function ReadStringDef(Name, Default: string): string; virtual;
    procedure ReadPref(Pref: TComponent); virtual;
    procedure WritePref(Pref: TComponent); virtual;
  published
    { Published declarations }
  end;

implementation

// Read a Boolean from the registry if it exists, otherwise return a default value.
function TRegistryDef.ReadBoolDef(Name: String; Default: Boolean): Boolean;
begin
    if ValueExists(Name) and (GetDataType(Name) = rdInteger) then
        Result := ReadBool(Name)
    else
        Result := Default;
end;

// Read an integer from the registry if it exists, otherwise return a default value.
function TRegistryDef.ReadIntegerDef(Name: String; Default: Integer): Integer;
begin
    if ValueExists(Name) and (GetDataType(Name) = rdInteger) then
        Result := ReadInteger(Name)
    else if ValueExists(Name) and (GetDataType(Name) = rdString) then
        Result := StrToIntDef(ReadString(Name), Default)
    else
        Result := Default;
end;

// Read a string from the registry if it exists, otherwise return a default value.
function TRegistryDef.ReadStringDef(Name, Default: string): string;
begin
    if ValueExists(Name) and (GetDataType(Name) in [rdString, rdExpandString]) then
        Result := ReadString(Name)
    else if ValueExists(Name) and (GetDataType(Name) = rdInteger) then
        Result := IntToStr(ReadInteger(Name))
    else
        Result := Default;
end;

// Read a string from the registry if it exists, otherwise return a default value.
procedure TRegistryDef.ReadPref(Pref: TComponent);
var Name: string;
begin
    if not (Pref is TControl) then Exit;

    Name := Trim(StringReplace((Pref as TControl).Name, '_', ' ', [rfReplaceAll]));

    if      (Pref is TCheckBox) then
            (Pref as TCheckBox).Checked := ReadBoolDef(Name, (Pref as TCheckBox).Checked)

    else if (Pref is TRadioButton) then
            (Pref as TRadioButton).Checked := ReadBoolDef(Name, (Pref as TRadioButton).Checked)

    else if (Pref is TRadioGroup) then
            (Pref as TRadioGroup).ItemIndex := ReadIntegerDef(Name, (Pref as TRadioGroup).ItemIndex)

    else if (Pref is TComboBox) then
            (Pref as TComboBox).ItemIndex := ReadIntegerDef(Name, (Pref as TComboBox).ItemIndex)

    else if (Pref is TEdit) then
            (Pref as TEdit).Text        := ReadStringDef(Name, (Pref as TEdit).Text)

    else if (Pref is TLabel) then
            (Pref as TLabel).Caption    := ReadStringDef(Name, (Pref as TLabel).Caption)

    else if (Pref is TSpinEdit) then
            (Pref as TSpinEdit).Value   := ReadIntegerDef(Name, (Pref as TSpinEdit).Value)

    else if (Pref is TPanel) then
            (Pref as TPanel).Color      := ReadIntegerDef(Name, (Pref as TPanel).Color);
end;

// Write a value to the registry from the Control passed in, based on the type of the Control.
procedure TRegistryDef.WritePref(Pref: TComponent);
var Name: string;
begin
    if not (Pref is TControl) then Exit;

    Name := Trim(StringReplace((Pref as TControl).Name, '_', ' ', [rfReplaceAll]));

    if      Pref is TCheckBox    then WriteBool(Name,    (Pref as TCheckBox).Checked)
    else if Pref is TRadioButton then WriteBool(Name,    (Pref as TRadioButton).Checked)
    else if Pref is TRadioGroup  then WriteInteger(Name, (Pref as TRadioGroup).ItemIndex)
    else if Pref is TComboBox    then WriteInteger(Name, (Pref as TComboBox).ItemIndex)
    else if Pref is TEdit        then WriteString(Name,  (Pref as TEdit).Text)
    else if Pref is TLabel       then WriteString(Name,  (Pref as TLabel).Caption)
    else if Pref is TSpinEdit    then try WriteInteger(Name, (Pref as TSpinEdit).Value); except; end
    else if Pref is TPanel       then WriteInteger(Name, (Pref as TPanel).Color);
end;

end.
