unit Scoreboard;

interface

uses SysUtils, TornTeamBox1;

type
//    THitArray = array [1..6] of Integer;
    TColourTeamArray = array [1..4] of Integer;  // 1 = red;  2 = grn;  3 = yel;  4 = blu

function  BuildTeamHandicap(TeamBox: TTeamBox): Integer;
function  FindTopThree(Team: TTeamBoxList; var ColourTeam: TColourTeamArray; Blue2, Blue3: Boolean): Boolean;

implementation

uses Dialogs, TornMain, Utility;

// Build information about a single team, for use on a single row of the scoreboard.
function BuildTeamHandicap(TeamBox: TTeamBox): Integer;
var TeamHandicap: String;
begin
    TeamHandicap := TeamBox.Handicap;
    if TeamHandicap = '0' then
        TeamHandicap := '100'
    else if Pos('+', TeamHandicap) = 1 then  // Show the team's + handicap.
        TeamHandicap := Copy(TeamHandicap, 2, Length(TeamHandicap))  // trim the leading "+" to keep StrToInt happy.
    else if Pos('-', TeamHandicap) = 1 then
        // do nothing
    else  // Show the team's % handicap
        TeamHandicap := Copy(TeamHandicap, 1, Length(TeamHandicap) - 1);  // trim the trailing "%"

    Result := StrToIntDef(TeamHandicap, 0);
    if (Result < -3276) or (Result > 3276) then Result := 0;  // make sure we don't go out the -32768..+32767 range allowed for scaled Accuracy.
end;

//       What teams will we show
// on the scoreboard's team score bars?
//       Answer in ColourTeam[].
//
//         And how many teams?
//    Return True for three or less;
//        False if four or more.
function FindTopThree(Team: TTeamBoxList; var ColourTeam: TColourTeamArray; Blue2, Blue3: Boolean): Boolean;
var i, j, k: Integer;
    NumTeams: Integer;
    TeamNum: array [1..3] of Integer;
    fit, bestfit: Real;

    // Count the number of players on this team that are of the specified colour.
    // 1 = red;  2 = grn;  3 = yel;  4 = blu.
    function FitColour(ThisTeam, colour: Integer): Integer;
    var i: Integer;
        count: Integer;
    begin
        count := 0;
        if ThisTeam > 0 then
            for i := 0 to Team[ThisTeam].Items.Count - 1 do
                if (Team[ThisTeam].Items[i].ImageIndex mod 4 = colour - 1) and
                   (Copy(Team[ThisTeam].Items[i].SubItems[tbMemberID], 1, 6) <> 'Adjust') then
                    Inc(count);

        Result := count;
    end;

begin  //---FindTopThree---
    NumTeams := 0;
    Result := True;
    for i := 1 to Team.Count - 1 do begin
        if Team[i].Rank in [1..3] then begin    // find the top three teams
            Inc(NumTeams);                      // remember their numbers in
            TeamNum[Team[i].Rank] := i;         //     the TeamNum array
        end;
        if Team[i].Rank = 4 then Result := False;
    end;
    if NumTeams = 0 then begin
        for i := 1 to 4 do
            ColourTeam[i] := -1;
        Exit;
    end;
    if NumTeams = 1 then TeamNum[2] := -1;      // there's less than three teams?
    if NumTeams in [1,2] then TeamNum[3] := -1; // flag the excess teams as bad:
                                                //     set them to minus one
    bestfit := 0.0;
    for i := 1 to 4 do      //           loop i, j and k
        for j := 1 to 4 do  // through red, green, yellow, and blue;
            if i <> j then  //          with all different
                for k := 1 to 4 do
                    if (i <> k) and (j <> k) then begin
                        fit := FitColour(TeamNum[1], i) * 1.02  //    calculate the fit
                             + FitColour(TeamNum[2], j) * 1.01  // biased in favour of the
                             + FitColour(TeamNum[3], k) * 1.00; //   top and second teams

                        if fit > bestfit then begin         //      remember always
                            bestfit := fit;                 // the best fit and also its
                            ColourTeam[i] := TeamNum[1];    //      matching i,j,k
                            ColourTeam[j] := TeamNum[2];
                            ColourTeam[k] := TeamNum[3];
                            ColourTeam[10 - i - j - k] := -1;
                        end;
                    end;

    if      Blue2 then ColourTeam[2] := ColourTeam[4]   //   if a best is blue,
    else if Blue3 then ColourTeam[3] := ColourTeam[4];  // copy it to grn or yel
                                                        //     appropriately
end;

end.
